# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from otcextensions.sdk.function_graph.v2 import function

from openstack import _log

from otcextensions.tests.functional.sdk.function_graph import TestFg

_logger = _log.setup_logging('openstack')


class TestFunctionInstances(TestFg):
    ID = None
    uuid = uuid.uuid4().hex[:8]

    def setUp(self):
        super(TestFunctionInstances, self).setUp()
        self.function = self.client.create_function(**TestFg.function_attrs)
        assert isinstance(self.function, function.Function)

        self.addCleanup(
            self.client.delete_function,
            self.function
        )

    def test_export(self):
        inst = self.client.export_function(self.function, type="code")
        self.assertIsNotNone(inst.file_name)
        self.assertIsNotNone(inst.code_url)

    def test_import(self):
        attrs = {
            'func_name': 'test',
            'file_type': 'zip',
            'file_name': 'test.zip',
            'file_code': 'UEsDBBQAAAAIAPBbPFpOs3AMAgEAAEwCAAAGAAAAZGVwLnB5jVHB'
                         'asMwDL37K4R3iSGMMtglsNNW2HH0B4oXq9SjtY2shJbSf5/tpl5y'
                         'GFQX23qS3nuWPQZPDD/ROyGEwR3stTMHpAZHdNxC7x3jiVUnIAXT'
                         '+XbJ8QRfmiIC7xGsCwND6YHGaNaqlhVom3PwVoieD16beCNQYj6O'
                         'bGrP4wL50Ro0kNtqRch4IzfYox0nsJPtjGExboM8kAMNhDF4F7Fi'
                         '90QSdKnJHDKy5iG+e4Oyg5fVql3C396cE7CTH9kOTUI6uPxJuMra'
                         'cp0RFinFvRmOITZ3CZNiPPUYGNblsD6pjoDzr/4sawEk8hRrvjy3'
                         'D9p5/d/OOs9JNiKnxasHLSzJlfgFUEsBAhQDFAAAAAgA8Fs8Wk6z'
                         'cAwCAQAATAIAAAYAAAAAAAAAAAAAAKSBAAAAAGRlcC5weVBLBQYA'
                         'AAAAAQABADQAAAAmAQAAAAA='
        }
        inst = self.client.import_function(**attrs)
        self.assertIsNotNone(inst.file_name)
        self.assertIsNotNone(inst.code_url)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack import exceptions


class FuncDestinationConfig(resource.Resource):
    # Object type.
    # OBS
    # SMN
    # FunctionGraph
    destination = resource.Body('destination', type=str)
    # Parameters (in JSON format) corresponding to the target service.
    param = resource.Body('param', type=str)


class FuncAsyncDestinationConfig(resource.Resource):
    on_success = resource.Body('on_success', type=FuncDestinationConfig)
    on_failure = resource.Body('on_failure', type=FuncDestinationConfig)


class Notification(resource.Resource):
    base_path = '/fgs/functions/%(function_urn)s/async-invoke-config'
    # Capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True
    allow_commit = True

    create_method = 'PUT'

    create_requires_id = False
    requires_id = False

    _query_mapping = resource.QueryParameters(
        'marker', 'limit',
    )
    # Properties
    function_urn = resource.URI('function_urn', type=str)

    # Attributes
    # Function URN.
    func_urn = resource.Body('func_urn', type=str)
    # Maximum validity period of a message. Value range: 1-86,400.
    # Unit: second.
    max_async_event_age_in_seconds = resource.Body(
        'max_async_event_age_in_seconds', type=int
    )
    max_async_retry_attempts = resource.Body(
        'max_async_retry_attempts', type=int
    )
    # Asynchronous invocation target.
    destination_config = resource.Body(
        'destination_config', type=FuncAsyncDestinationConfig
    )
    # Time when asynchronous execution notification was configured.
    created_at = resource.Body('created_time', type=str)
    # Time when the asynchronous execution notification settings
    # were last modified.
    updated_at = resource.Body('last_modified', type=str)
    # Whether to enable asynchronous invocation status persistence.
    enable_async_status_log = resource.Body(
        'enable_async_status_log', type=str
    )


class Requests(resource.Resource):
    base_path = '/fgs/functions/%(function_urn)s/async-invocations'
    resources_key = 'invocations'

    _query_mapping = resource.QueryParameters(
        'request_id', 'marker', 'limit',
        'status', 'query_begin_time', 'query_end_time'
    )

    # Capabilities
    allow_list = True
    allow_create = True

    # Properties
    function_urn = resource.URI('function_urn', type=str)
    # Stop mode.
    # Enumeration values:
    # force
    # recursive
    type = resource.Body('type', type=str)

    # Attributes
    # Asynchronous invocation request ID.
    request_id = resource.Body('request_id', type=str)
    # Asynchronous invocation status. Options:
    # WAIT
    # RUNNING
    # SUCCESS
    # FAIL
    # DISCARD
    status = resource.Body('status', type=str)
    # Asynchronous invocation error information.
    # If the execution is successful, no value is returned.
    error_message = resource.Body('error_message', type=str)
    # Asynchronous invocation error code.
    # If the execution is successful, 0 is returned.
    error_code = resource.Body('error_code', type=int)
    # Start time of the asynchronous invocation.
    # The format is "YYYY-MM-DD'T'HH:mm:ss" (UTC time).
    start_time = resource.Body('start_time', type=str)
    # End time of the asynchronous invocation.
    # The format is "YYYY-MM-DD'T'HH:mm:ss" (UTC time).
    end_time = resource.Body('end_time', type=str)

    def _stop(self, session, function_urn, **attrs):
        """Stop asynchronous invocation of a function
        """
        url = ('fgs/functions/%(function_urn)s/cancel' %
               {'function_urn': function_urn})
        response = session.post(url, json=attrs)
        exceptions.raise_from_response(response)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''SDRS Active-active domains CLI implementation'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_domain(obj):
    """Flatten the structure of active-active domains into a single dict
    """
    obj = obj.domains
    i = 0
    data = {
        'id': obj[i].id,
        'name': obj[i].name,
        'description': obj[i].description,
        'sold_out': obj[i].sold_out,
        'local_replication_cluster':
            obj[i].local_replication_cluster.availability_zone,
        'remote_replication_cluster':
            obj[i].remote_replication_cluster.availability_zone
    }

    return data


class ListDomain(command.Lister):
    _description = _('List Active-active domains')
    columns = (
        'ID',
        'name',
        'description',
        'sold_out',
        'local_replication_cluster',
        'remote_replication_cluster'
    )

    def get_parser(self, prog_name):
        parser = super(ListDomain, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.sdrs

        data = client.get_domains()

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_domain(s), self.columns,
                 ) for s in data))
        return table



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
from openstack import exceptions
from openstack import proxy as os_proxy

_logger = _log.setup_logging('openstack')


class Proxy(os_proxy.Proxy):

    def _find(self, resource_type, name_or_id, ignore_missing=True,
              endpoint_override=None, headers=None, requests_auth=None,
              **attrs):
        """Find a resource

        :param name_or_id: The name or ID of a resource to find.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~openstack.resource.Resource.find`
                           method, such as query parameters.

        :returns: An instance of ``resource_type`` or None
        """
        result = resource_type.find(self, name_or_id,
                                    ignore_missing=ignore_missing,
                                    endpoint_override=endpoint_override,
                                    headers=headers,
                                    requests_auth=requests_auth,
                                    **attrs)
        # Inject endpoint_override into the resource for potential
        # direct use (i.e. instance.reboot)
        if endpoint_override:
            setattr(result, 'endpoint_override', endpoint_override)

        return result

    def _delete(self, resource_type, value, ignore_missing=True,
                endpoint_override=None, headers=None, requests_auth=None,
                **attrs):
        """Delete a resource

        :param resource_type: The type of resource to delete. This should
            be a :class:`~openstack.resource.Resource`
            subclass with a ``from_id`` method.
        :param value: The value to delete. Can be either the ID of a
            resource or a :class:`~openstack.resource.Resource`
            subclass.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent resource.
        :param dict attrs: Attributes to be passed onto the
            :meth:`~openstack.resource.Resource.delete`
            method, such as the ID of a parent resource.

        :returns: The result of the ``delete``
        :raises: ``ValueError`` if ``value`` is a
            :class:`~openstack.resource.Resource` that doesn't match
            the ``resource_type``.
            :class:`~openstack.exceptions.ResourceNotFound` when
            ignore_missing if ``False`` and a nonexistent resource
            is attempted to be deleted.

        """
        res = self._get_resource(resource_type, value, **attrs)

        try:
            rv = res.delete(
                self,
                error_message=(
                    "Unable to delete {resource_type} for {value}".format(
                        resource_type=resource_type.__name__,
                        value=value,
                    )
                ),
                endpoint_override=endpoint_override,
                headers=headers,
                requests_auth=requests_auth
            )
        except exceptions.NotFoundException:
            if ignore_missing:
                return None
            raise

        return rv

    def _update(self, resource_type, value,
                endpoint_override=None, headers=None, requests_auth=None,
                **attrs):
        """Update a resource

        :param resource_type: The type of resource to update.
        :type resource_type: :class:`~openstack.resource.Resource`
        :param value: The resource to update. This must either be a
                      :class:`~openstack.resource.Resource` or an id
                      that corresponds to a resource.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~openstack.resource.Resource.update`
                           method to be updated. These should correspond
                           to either :class:`~openstack.resource.Body`
                           or :class:`~openstack.resource.Header`
                           values on this resource.

        :returns: The result of the ``update``
        :rtype: :class:`~openstack.resource.Resource`
        """
        res = self._get_resource(resource_type, value, **attrs)
        return res.update(
            self,
            endpoint_override=endpoint_override,
            headers=headers,
            requests_auth=requests_auth
        )

    def _create(self, resource_type,
                endpoint_override=None, headers=None, requests_auth=None,
                prepend_key=True,
                **attrs):
        """Create a resource from attributes

        :param resource_type: The type of resource to create.
        :type resource_type: :class:`~openstack.resource.Resource`
        :param path_args: A dict containing arguments for forming the request
                          URL, if needed.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~openstack.resource.Resource.create`
                           method to be created. These should correspond
                           to either :class:`~openstack.resource.Body`
                           or :class:`~openstack.resource.Header`
                           values on this resource.

        :returns: The result of the ``create``
        :rtype: :class:`~openstack.resource.Resource`
        """
        res = resource_type.new(**attrs)
        persist = res.create(
            self,
            endpoint_override=endpoint_override,
            headers=headers,
            requests_auth=requests_auth,
            prepend_key=prepend_key,
        )

        # Inject endpoint_override into the resource for potential
        # direct use (i.e. instance.reboot)
        if endpoint_override:
            persist.endpoint_override = endpoint_override

        return persist

    def _get(self, resource_type, value=None, requires_id=True,
             endpoint_override=None, headers=None, requests_auth=None,
             **attrs):
        """Get a resource

        :param resource_type: The type of resource to get.
        :type resource_type: :class:`~openstack.resource.Resource`
        :param value: The value to get. Can be either the ID of a
                      resource or a :class:`~openstack.resource.Resource`
                      subclass.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~openstack.resource.Resource.get`
                           method. These should correspond
                           to either :class:`~openstack.resource.Body`
                           or :class:`~openstack.resource.Header`
                           values on this resource.

        :returns: The result of the ``get``
        :rtype: :class:`~openstack.resource.Resource`
        """
        res = self._get_resource(resource_type, value, **attrs)

        persist = res.get(
            self, requires_id=requires_id,
            error_message="No {resource_type} found for {value}".format(
                resource_type=resource_type.__name__, value=value),
            endpoint_override=endpoint_override,
            headers=headers,
            requests_auth=requests_auth,
        )

        # Inject endpoint_override into the resource for potential
        # direct use (i.e. instance.reboot)
        if endpoint_override:
            persist.endpoint_override = endpoint_override

        return persist

    def _list(self, resource_type, value=None, paginated=False,
              endpoint_override=None, headers=None, requests_auth=None,
              **attrs):
        """List a resource

        :param resource_type: The type of resource to delete. This should
                              be a :class:`~openstack.resource.Resource`
                              subclass with a ``from_id`` method.
        :param value: The resource to list. It can be the ID of a resource, or
                      a :class:`~openstack.resource.Resource` object. When set
                      to None, a new bare resource is created.
        :param bool paginated: When set to ``False``, expect all of the data
                               to be returned in one response. When set to
                               ``True``, the resource supports data being
                               returned across multiple pages.
        :param dict attrs: Attributes to be passed onto the
            :meth:`~openstack.resource.Resource.list` method. These should
            correspond to either :class:`~openstack.resource.URI` values
            or appear in :data:`~openstack.resource.Resource._query_mapping`.

        :returns: A generator of Resource objects.
        :raises: ``ValueError`` if ``value`` is a
                 :class:`~openstack.resource.Resource` that doesn't match
                 the ``resource_type``.
        """
        res = self._get_resource(resource_type, value, **attrs)
        return res.list(self, paginated=paginated,
                        endpoint_override=endpoint_override,
                        headers=headers,
                        requests_auth=requests_auth,
                        **attrs)

    def _head(self, resource_type, value=None,
              endpoint_override=None, headers=None, requests_auth=None,
              **attrs):
        """Retrieve a resource's header

        :param resource_type: The type of resource to retrieve.
        :type resource_type: :class:`~openstack.resource.Resource`
        :param value: The value of a specific resource to retreive headers
                      for. Can be either the ID of a resource,
                      a :class:`~openstack.resource.Resource` subclass,
                      or ``None``.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~openstack.resource.Resource.head` method.
                           These should correspond to
                           :class:`~openstack.resource.URI` values.

        :returns: The result of the ``head`` call
        :rtype: :class:`~openstack.resource.Resource`
        """
        res = self._get_resource(resource_type, value, **attrs)
        return res.head(self,
                        endpoint_override=endpoint_override,
                        headers=headers,
                        requests_auth=requests_auth,
                        )



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create a RDS instance
"""
import openstack


openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


attrs = {
    "name": "rds_name",
    "port": 3306,
    "ha": {
        "mode": "Ha",
        "replication_mode": "async"
    },
    "region": "eu-de",
    "datastore": {
        "type": "MySQL",
        "version": "8.0"
    },
    "volume": {
        "type": "ULTRAHIGH",
        "size": 40
    },
    "password": "admin_password",
    "private_ips": [],
    "public_ips": [],
    "db_user_name": "root",
    "availability_zone": "eu-de-01,eu-de-02",
    "vpc_id": "vpc_id",
    "subnet_id": "network_id_of_the_subnet",
    "security_group_id": "secgrp_id",
    "flavor_ref": "rds.mysql.c2.medium.ha",
    "switch_strategy": "reliability",
    "backup_strategy": {
        "start_time": "23:00-00:00",
        "keep_days": 10
    },
    "charge_info": {
        "charge_mode": "postPaid"
    }
}

instance = conn.rds.create_instance(**attrs)
print(instance)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import backend_server

EXAMPLE_MEMBER = {
    'gateway_id': 'gw-123',
    'vpc_chan_id': 'vpc-456',
    'host': '192.168.1.10',
    'weight': 20,
    'is_backup': True,
    'member_group_name': 'group-A',
    'status': 1,
    'port': 8080,
    'ecs_id': 'ecs-123',
    'ecs_name': 'ecs-node-A',
    'id': 'mem-001',
    'vpc_channel_id': 'vpc-456',
    'create_time': '2025-01-01T12:00:00Z',
    'member_group_id': 'mg-001'
}


class TestBackendServer(base.TestCase):

    def test_basic(self):
        sot = backend_server.BackendServer()
        self.assertEqual(
            'apigw/instances/%(gateway_id)s/vpc-channels/%(vpc_chan_id)s/'
            'members',
            sot.base_path
        )
        self.assertEqual('members', sot.resources_key)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = backend_server.BackendServer(**EXAMPLE_MEMBER)
        self.assertEqual(EXAMPLE_MEMBER['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_MEMBER['vpc_chan_id'], sot.vpc_chan_id)
        self.assertEqual(EXAMPLE_MEMBER['host'], sot.host)
        self.assertEqual(EXAMPLE_MEMBER['weight'], sot.weight)
        self.assertEqual(EXAMPLE_MEMBER['is_backup'], sot.is_backup)
        self.assertEqual(EXAMPLE_MEMBER['member_group_name'],
                         sot.member_group_name)
        self.assertEqual(EXAMPLE_MEMBER['status'], sot.status)
        self.assertEqual(EXAMPLE_MEMBER['port'], sot.port)
        self.assertEqual(EXAMPLE_MEMBER['ecs_id'], sot.ecs_id)
        self.assertEqual(EXAMPLE_MEMBER['ecs_name'], sot.ecs_name)
        self.assertEqual(EXAMPLE_MEMBER['id'], sot.id)
        self.assertEqual(EXAMPLE_MEMBER['vpc_channel_id'], sot.vpc_channel_id)
        self.assertEqual(EXAMPLE_MEMBER['create_time'], sot.create_time)
        self.assertEqual(EXAMPLE_MEMBER['member_group_id'],
                         sot.member_group_id)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
# import datetime
import random
import uuid

import mock

from keystoneauth1 import adapter

from otcextensions.tests.unit.osclient import test_base

from otcextensions.common import sdk_utils
from otcextensions.osclient.deh.v1 import host as _host

from otcextensions.sdk.deh.v1 import host
from otcextensions.sdk.deh.v1 import host_type
from otcextensions.sdk.deh.v1 import server


def gen_data(data, columns):
    """Fill expected data tuple based on columns list
    """
    result = []
    for attr in columns:
        _data = getattr(data, attr, '')
        if attr == 'host_properties':
            result.append(_host.HostPropertiesFormatter(_data))
        elif attr == 'tags':
            result.append(sdk_utils.ListOfDictColumn(_data))
        else:
            result.append(_data)
    return tuple(result)


class TestDeH(test_base.TestCommand):

    def setUp(self):
        super(TestDeH, self).setUp()

        self.app.client_manager.deh = mock.Mock()
        self.client = self.app.client_manager.deh
        self.client.session = mock.Mock(spec=adapter.Adapter)
        self.client.get = mock.Mock()


class FakeHost(test_base.Fake):
    """Fake one or more Host"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            # 'dedicated_host_ids': ['id-' + uuid.uuid4().hex],
            'name': uuid.uuid4().hex,
            'auto_placement': random.choice(['on', 'off']),
            'availability_zone': uuid.uuid4().hex,
            'state': random.choice(['available', 'fault', 'released']),
            'project_id': uuid.uuid4().hex,
            'available_vcpus': random.randint(1, 600),
            'available_memory': random.randint(1, 600),
            'instance_total': random.randint(1, 600),
            'allocated_at': uuid.uuid4().hex,
            'released_at': uuid.uuid4().hex,
            'host_properties': {
                'vcpus': random.randint(1, 600),
                'cores': random.randint(1, 600),
                'sockets': random.randint(1, 600),
                'memory': random.randint(1, 600),
                'host_type': uuid.uuid4().hex,
                'host_type_name': uuid.uuid4().hex,
                'available_instance_capacities': [
                    {'flavor': uuid.uuid4().hex}
                ],
            },
            'tags': [{'key': uuid.uuid4().hex, 'value': uuid.uuid4().hex}]

        }
        obj = host.Host.existing(**object_info)
        return obj


class FakeHostType(test_base.Fake):
    """Fake one or more Host type"""

    @classmethod
    def generate(cls):
        object_info = {
            'host_type': uuid.uuid4().hex,
            'host_type_name': uuid.uuid4().hex,
        }
        obj = host_type.HostType.existing(**object_info)
        return obj


class FakeServer(test_base.Fake):
    """Fake one or more HostServers"""

    @classmethod
    def generate(cls):
        object_info = {
            'addresses': {
                'id-' + uuid.uuid4().hex: [{
                    'addr': 'addr-' + uuid.uuid4().hex,
                    'version': 4
                }]
            },
            'created_at': uuid.uuid4().hex,
            'updated_at': uuid.uuid4().hex,
            'flavor': {'id-' + uuid.uuid4().hex},
            'id': 'id-' + uuid.uuid4().hex,
            'metadata': {
                'os_type': uuid.uuid4().hex
            },
            'name': uuid.uuid4().hex,
            'status': uuid.uuid4().hex,
            'tenant_id': uuid.uuid4().hex,
            'user_id': uuid.uuid4().hex
        }
        obj = server.Server.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Domain(resource.Resource):
    base_path = 'manage/namespaces/%(namespace)s/' \
                'repositories/%(repository)s/access-domains'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_commit = True
    allow_fetch = True

    commit_method = "PATCH"

    #: Organization namespace
    #: *Type:str*
    namespace = resource.URI('namespace')
    #: Image repository name
    #: *Type:str*
    repository = resource.URI('repository')
    #: Name of the account used for image sharing
    #: *Type:str*
    access_domain = resource.Body('access_domain', type=str)
    #: Currently, only the read permission is supported
    #: *Type:str*
    permit = resource.Body('permit', type=str)
    #: Valid until (UTC). If the sharing is permanent, the value is forever.
    #: Otherwise, the sharing is valid until 00:00:00 of the next day
    #: *Type:str*
    deadline = resource.Body('deadline', type=str)
    #: Description
    #: *Type:str*
    description = resource.Body('description', type=str)
    #: Creator ID
    #: *Type:str*
    creator_id = resource.Body('creator_id')
    #: Name of the creator
    #: *Type:str*
    creator_name = resource.Body('creator_name')
    #: Time when an image is created. It is the UTC standard time
    #: *Type:str*
    created_at = resource.Body('created')
    #: Time when an image is updated. It is the UTC standard time
    #: *Type:int*
    updated_at = resource.Body('updated')
    #: Status
    #: *Type:bool*
    status = resource.Body('status')
    #: Whether account exist
    #: *Type:bool*
    exist = resource.Body('exist')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log


from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestCleanup(base.BaseFunctionalTest):
    def setUp(self):
        super(TestCleanup, self).setUp()

    def test_01_delete(self):
        gateways = list(self.conn.nat.gateways())
        self.conn.nat._service_cleanup(dry_run=False)
        gateways = list(self.conn.nat.gateways())
        self.assertEqual(len(gateways), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.smn.v2 import template


EXAMPLE = {
    "message_template_name": "confirm_message",
    "protocol": "https",
    "update_time": "2016-08-02T08:22:25Z",
    "create_time": "2016-08-02T08:22:20Z",
    "request_id": "ba79ca8f794f4f50985ce7b98a401b47",
    "tag_names": [
        "topic_id_id4"
    ],
    "content": "(1/24)You are invited to subscribe to topic({topic_id_id4}).",
    "message_template_id": "57ba8dcecda844878c5dd5815b65d10f"
}


class TestTemplate(base.TestCase):

    def test_basic(self):
        sot = template.Template()
        self.assertEqual('message_templates', sot.resources_key)
        path = '/notifications/message_template'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = template.Template(**EXAMPLE)
        self.assertEqual(EXAMPLE['message_template_name'],
                         sot.name)
        self.assertEqual(EXAMPLE['message_template_id'],
                         sot.id)
        self.assertEqual(EXAMPLE['create_time'], sot.create_time)
        self.assertEqual(EXAMPLE['update_time'], sot.update_time)
        self.assertEqual(EXAMPLE['tag_names'], sot.tag_names)
        self.assertEqual(EXAMPLE['content'], sot.content)
        self.assertEqual(EXAMPLE['protocol'], sot.protocol)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Direct Connection v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListDirectConnections(command.Lister):

    _description = _("List of Direct Connections.")
    columns = (
        'id',
        'name',
        'port type',
        'provider',
        'bandwidth',
        'location',
        'status'
    )

    def get_parser(self, prog_name):
        parser = super(ListDirectConnections, self).get_parser(prog_name)
        parser.add_argument(
            '--id',
            metavar='<id>',
            help=_("Specifies the ID of the Direct Connection.")
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specified the name of Direct Connection.")
        )
        parser.add_argument(
            '--port_type',
            metavar='<port_type>',
            help=_("Specified the port type of Direct Connection. The value "
                   "can be 1G or 10G.")
        )
        parser.add_argument(
            '--bandwidth',
            metavar='<bandwidth>',
            type=int,
            help=_("Specified the bandwidth of Direct Connection in Mbit/s.")
        )
        parser.add_argument(
            '--location',
            metavar='<location>',
            help=_("Specified the access location of Direct Connection.")
        )
        parser.add_argument(
            '--peer_location',
            metavar='<peer_location>',
            help=_("Specifies the location of the on-premises facility at "
                   "the other end of the connection, specific to the street "
                   "or data center name.")
        )
        parser.add_argument(
            '--device_id',
            metavar='<device_id>',
            help=_("Specifies the gateway device ID of the Direct Connection.")
        )
        parser.add_argument(
            '--interface_name',
            metavar='<interface_name>',
            help=_("Specifies the name of the interface accessed by the "
                   "Direct Connection.")
        )
        parser.add_argument(
            '--redundant_id',
            metavar='<redundant_id>',
            help=_("Specifies the ID of the redundant connection using "
                   "the same gateway.")
        )
        parser.add_argument(
            '--provider',
            metavar='<provider>',
            help=_("Specifies the carrier who provides the leased line.")
        )
        parser.add_argument(
            '--provider_status',
            metavar='<provider_status>',
            help=_("Specifies the status of the carrier's leased line."
                   " The value can be ACTIVE or DOWN.")
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_("Specifies the connection type. The value can be hosted.")
        )
        parser.add_argument(
            '--hosting_id',
            metavar='<hosting_id>',
            help=_("Specifies the ID of the operations connection on which"
                   " the hosted connection is created.")
        )
        parser.add_argument(
            '--vlan',
            metavar='<vlan>',
            help=_("Specifies the VLAN pre-allocated to the hosted"
                   " connection.")
        )
        parser.add_argument(
            '--charge_mode',
            metavar='<charge_mode>',
            help=_("Specifies the billing mode. The value can be prepayment,"
                   " bandwidth, or traffic.")
        )
        parser.add_argument(
            '--apply_time',
            metavar='<apply_time>',
            help=_("Specifies the time when the connection is requested.")
        )
        parser.add_argument(
            '--create_time',
            metavar='<create_time>',
            help=_("Specifies the time when the connection is created.")
        )
        parser.add_argument(
            '--delete_time',
            metavar='<delete_time>',
            help=_("Specifies the time when the connection is deleted.")
        )
        parser.add_argument(
            '--order_id',
            metavar='<order_id>',
            help=_("Specifies the order number of the connection.")
        )
        parser.add_argument(
            '--product_id',
            metavar='<product_id>',
            help=_("Specifies the product ID corresponding to the "
                   "connection's order.")
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_("Specifies the connection status. The value can be ACTIVE, "
                   "DOWN, BUILD, ERROR, PENDING_DELETE, DELETED, APPLY, DENY, "
                   "PENDING_PAY, PAID, ORDERING, ACCEPT, or REJECTED.")
        )
        parser.add_argument(
            '--admin_state_up',
            metavar='<admin_state_up>',
            help=_("Specifies the administrative status of the connection."
                   "The value can be true or false.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas

        args_list = [
            'id',
            'name',
            'port_type',
            'bandwidth',
            'location',
            'peer_location',
            'device_id',
            'interface_name',
            'redundant_id',
            'provider',
            'provider_status',
            'type',
            'hosting_id',
            'vlan',
            'charge_mode',
            'apply_time',
            'create_time',
            'delete_time',
            'order_id',
            'product_id',
            'status',
            'admin_state_up'
        ]

        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.connections(**attrs)

        table = (self.columns, (utils.get_dict_properties(s, self.columns)
                                for s in data))
        return table


class ShowDirectConnection(command.ShowOne):
    _description = _("Show Direct Connection details.")

    def get_parser(self, prog_name):
        parser = super(ShowDirectConnection, self).get_parser(prog_name)
        parser.add_argument(
            'direct_connection',
            metavar='<direct_connection>',
            help=_("Specifies the connection ID or name.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas
        obj = client.find_connection(parsed_args.direct_connection)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class CreateDirectConnection(command.ShowOne):
    _description = _("Create new Direct Connection")

    def get_parser(self, prog_name):
        parser = super(CreateDirectConnection, self).get_parser(prog_name)
        parser.add_argument(
            'port_type',
            metavar='<port_type>',
            help=_("Specified the port type of Direct Connection. The value "
                   "can be 1G or 10G.")
        )
        parser.add_argument(
            'bandwidth',
            metavar='<bandwidth>',
            type=int,
            help=_("Specified the bandwidth of Direct Connection in Mbit/s.")
        )
        parser.add_argument(
            'location',
            metavar='<location>',
            help=_("Specified the access location of Direct Connection.")
        )
        parser.add_argument(
            'provider',
            metavar='<provider>',
            help=_("Specifies the carrier who provides the leased line.")
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specified the name of Direct Connection.")
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary information about the connection.")
        )
        parser.add_argument(
            '--peer_location',
            metavar='<peer_location>',
            help=_("Specifies the location of the on-premises facility at "
                   "the other end of the connection, specific to the street "
                   "or data center name.")
        )
        parser.add_argument(
            '--device_id',
            metavar='<device_id>',
            help=_("Specifies the gateway device ID of the Direct Connection.")
        )
        parser.add_argument(
            '--interface_name',
            metavar='<interface_name>',
            help=_("Specifies the name of the interface accessed by the "
                   "Direct Connection.")
        )
        parser.add_argument(
            '--redundant_id',
            metavar='<redundant_id>',
            help=_("Specifies the ID of the redundant connection using "
                   "the same gateway.")
        )
        parser.add_argument(
            '--provider_status',
            metavar='<provider_status>',
            help=_("Specifies the status of the carrier's leased line. "
                   "The value can be ACTIVE or DOWN.")
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_("Specifies the connection type. The value can be hosted.")
        )
        parser.add_argument(
            '--hosting_id',
            metavar='<hosting_id>',
            help=_("Specifies the ID of the operations connection on which "
                   "the hosted connection is created.")
        )
        parser.add_argument(
            '--vlan',
            metavar='<vlan>',
            type=int,
            help=_("Specifies the VLAN pre-allocated to the hosted "
                   "connection.")
        )
        parser.add_argument(
            '--charge_mode',
            metavar='<charge_mode>',
            help=_("Specifies the billing mode. The value can be prepayment, "
                   "bandwidth, or traffic.")
        )
        parser.add_argument(
            '--order_id',
            metavar='<order_id>',
            help=_("Specifies the order number of the connection.")
        )
        parser.add_argument(
            '--product_id',
            metavar='<product_id>',
            help=_("Specifies the product ID corresponding to the "
                   "connection's order.")
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_("Specifies the connection status. The value can be ACTIVE, "
                   "DOWN, BUILD, ERROR, PENDING_DELETE, DELETED, APPLY, DENY, "
                   "PENDING_PAY, PAID, ORDERING, ACCEPT, or REJECTED.")
        )
        parser.add_argument(
            '--admin_state_up',
            metavar='<admin_state_up>',
            type=bool,
            help=_("Specifies the administrative status of the connection. "
                   "The value can be true or false.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas

        args_list = [
            'name',
            'description',
            'port_type',
            'bandwidth',
            'location',
            'peer_location',
            'device_id',
            'interface_name',
            'redundant_id',
            'provider',
            'provider_status',
            'type',
            'hosting_id',
            'vlan',
            'charge_mode',
            'order_id',
            'product_id',
            'status',
            'admin_state_up'
        ]
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        obj = client.create_connection(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class UpdateDirectConnection(command.ShowOne):
    _description = _("Update a Direct Connection")

    def get_parser(self, prog_name):
        parser = super(UpdateDirectConnection, self).get_parser(prog_name)
        parser.add_argument(
            'direct_connection',
            metavar='<direct_connection>',
            help=_("Specifies the connection ID or name.")
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specifies the connection name.")
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary information about the connection.")
        )
        parser.add_argument(
            '--bandwidth',
            metavar='<bandwidth>',
            type=int,
            help=_("Specifies the bandwidth of the connection in Mbit/s. "
                   "The value can be 1G or 10G.")
        )
        parser.add_argument(
            '--provider_status',
            metavar='<provider_status>',
            help=_("Specifies the status of the carrier's leased line. "
                   "The value can be ACTIVE or DOWN.")
        )
        parser.add_argument(
            '--order_id',
            metavar='<order_id>',
            help=_("Specifies the order number of the connection.")
        )
        parser.add_argument(
            '--product_id',
            metavar='<product_id>',
            help=_("Specifies the product ID corresponding to the "
                   "connection's order.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas

        args_list = [
            'name',
            'description',
            'bandwidth',
            'provider_status',
            'order_id',
            'product_id'
        ]
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        if parsed_args.direct_connection:
            direct_connection = client.find_connection(
                parsed_args.direct_connection
            )
            obj = client.update_connection(
                direct_connection.id, **attrs
            )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteDirectConnection(command.Command):
    _description = _("Delete the Direct Connection.")

    def get_parser(self, prog_name):
        parser = super(DeleteDirectConnection, self).get_parser(prog_name)
        parser.add_argument(
            'direct_connection',
            metavar='<direct_connection>',
            help=_("Direct Connection to delete.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas
        if parsed_args.direct_connection:
            direct_connection = client.find_connection(
                parsed_args.direct_connection)
            client.delete_connection(direct_connection.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from keystoneauth1 import adapter
import mock
from openstack.tests.unit import base

from otcextensions.sdk.sdrs.v1 import replication_pair as _replication_pair


EXAMPLE = {
    'id': uuid.uuid4(),
    'name': uuid.uuid4(),
    'description': uuid.uuid4(),
    'replication_model': 'hypermetro',
    'status': 'protected',
    'volume_ids': uuid.uuid4(),
    'server_group_id': uuid.uuid4(),
    'priority_station': 'source',
    'progress': 100,
    'replication_status': 'active',
    'fault_level': 0,
    'failure_detail': None,
    'created_at': '2022-03-05 09:29:16.221',
    'updated_at': '2022-03-05 13:27:02.612',
    'record_metadata': {
        'volume_size': 100,
        'volume_type': "SAS",
        'multiattach': False,
        'bootable': False
    },
    'attachment': [
        {
            'protected_instance': uuid.uuid4(),
            'device': '/dev/vda'
        }
    ]
}


class TestProtectedInstance(base.TestCase):

    def setUp(self):
        super(TestProtectedInstance, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sot = _replication_pair.ReplicationPair()

    def test_basic(self):
        sot = _replication_pair.ReplicationPair()
        self.assertEqual('replication', sot.resource_key)
        self.assertEqual('replications', sot.resources_key)
        self.assertEqual('/replications',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertDictEqual({
            'availability_zone': 'availability_zone',
            'limit': 'limit',
            'marker': 'marker',
            'name': 'name',
            'offset': 'offset',
            'protected_instance_id': 'protected_instance_id',
            'protected_instance_ids': 'protected_instance_ids',
            'query_type': 'query_type',
            'server_group_id': 'server_group_id',
            'server_group_ids': 'server_group_ids',
            'status': 'status'
        }, sot._query_mapping._mapping)

    def test_make_it(self):
        test_replication_pair = _replication_pair.ReplicationPair(**EXAMPLE)
        self.assertEqual(
            EXAMPLE['id'],
            test_replication_pair.id)
        self.assertEqual(
            EXAMPLE['name'],
            test_replication_pair.name)
        self.assertEqual(
            EXAMPLE['description'],
            test_replication_pair.description)
        self.assertEqual(
            EXAMPLE['replication_model'],
            test_replication_pair.replication_model)
        self.assertEqual(
            EXAMPLE['status'],
            test_replication_pair.status)
        self.assertEqual(
            EXAMPLE['volume_ids'],
            test_replication_pair.volume_ids)
        self.assertEqual(
            EXAMPLE['server_group_id'],
            test_replication_pair.server_group_id)
        self.assertEqual(
            EXAMPLE['priority_station'],
            test_replication_pair.priority_station)
        self.assertEqual(
            EXAMPLE['progress'],
            test_replication_pair.progress)
        self.assertEqual(
            EXAMPLE['replication_status'],
            test_replication_pair.replication_status)
        self.assertEqual(
            EXAMPLE['fault_level'],
            test_replication_pair.fault_level)
        self.assertEqual(
            EXAMPLE['failure_detail'],
            test_replication_pair.failure_detail)
        self.assertEqual(
            EXAMPLE['created_at'],
            test_replication_pair.created_at)
        self.assertEqual(
            EXAMPLE['updated_at'],
            test_replication_pair.updated_at)
        self.assertEqual(
            EXAMPLE['record_metadata']['volume_size'],
            test_replication_pair.record_metadata.volume_size)
        self.assertEqual(
            EXAMPLE['record_metadata']['volume_type'],
            test_replication_pair.record_metadata.volume_type)
        self.assertEqual(
            EXAMPLE['record_metadata']['multiattach'],
            test_replication_pair.record_metadata.multiattach)
        self.assertEqual(
            EXAMPLE['record_metadata']['bootable'],
            test_replication_pair.record_metadata.bootable)
        self.assertEqual(
            EXAMPLE['attachment'][0]['protected_instance'],
            test_replication_pair.attachment[0].protected_instance)
        self.assertEqual(
            EXAMPLE['attachment'][0]['device'],
            test_replication_pair.attachment[0].device)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils

from otcextensions.i18n import _
from otcextensions.sdk.auto_scaling.v1 import _base


class Instance(_base.Resource):
    ACTION_TYPES = ['ADD', 'REMOVE', 'PROTECT', 'UNPROTECT']

    resource_key = 'scaling_group_instance'
    resources_key = 'scaling_group_instances'
    # ok, we just fix the base path to list because there are no common rules
    # for the operations for instance
    base_path = '/scaling_group_instance'
    query_marker_key = 'start_number'

    # capabilities
    allow_list = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'id', 'name',
        'health_status', 'limit',
        scaling_group_id='group_id',
        lifecycle_status='life_cycle_state',
        marker=query_marker_key
    )

    #: Properties
    #: AutoScaling instance id
    id = resource.Body('instance_id', alternate_id=True)
    #: AutoScaling instance name
    name = resource.Body('instance_name')
    #: Id of AutoScaling group the instance belongs to
    scaling_group_id = resource.URI('scaling_group_id')
    #: Name of AutoScaling group the instance belongs to
    scaling_group_name = resource.Body('scaling_group_name')
    #: Id of AutoScaling config the instance create with
    scaling_configuration_id = resource.Body('scaling_configuration_id')
    #: Name of AutoScaling config the instance create with
    scaling_configuration_name = resource.Body('scaling_configuration_name')
    #: AutoScaling instance lifecycle state, valid values include:
    #: ``INSERVICE``, ``PENDING``, ``REMOVING``
    lifecycle_state = resource.Body('life_cycle_state')
    #: AutoScaling instance health state, valid values include:
    #: ``INITIALIZING``, ``NORMAL``, ``ERROR``
    health_status = resource.Body('health_status')
    #: AutoScaling instance create time
    create_time = resource.Body('create_time')

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        """Find a resource by its name or id.

        This method was added in order to be able to get an object of the
        current class if it exists. This object will be used in
        some other proxy methods.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~openstack.resource.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        session = cls._get_session(session)
        group_id = params.pop('group_id', None)

        base_path = '/scaling_group_instance/{id}/list'.format(id=group_id)
        data = cls.list(session, base_path=base_path, **params)
        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))

    def remove(self, session, delete_instance=False, ignore_missing=True):
        """Remove an instance of auto scaling group

        :precondition:
            * the instance must in ``INSERVICE`` status.
            * after remove the instance number of auto scaling group should not
                be less than min instance number.
            * The owner auto scaling group should not in scaling status.

        :param session: openstack session
        :param bool delete_instance: When set to ``True``, instance will be
            deleted after removed.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the config does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent config.

        :returns: None
       """
        uri = utils.urljoin('/scaling_group_instance', self.id)
        delete_instance = 'yes' if delete_instance else 'no'
        return session.delete(uri,
                              params={'instance_delete': delete_instance})

    def _action(self, session, body):
        """Preform alarm actions given the message body.

        """
        url = utils.urljoin(self.base_path, self.scaling_group_id, 'action')
        return session.post(
            url,
            # endpoint_override=endpoint_override,
            json=body)

    def batch_action(self, session, instances, action, delete_instance=False):
        """batch action on auto-scaling instances

        make sure all configs should not been used by auto-scaling group

        :param session: openstack session
        :param instances: The list item value can be the ID of an instance
            or a :class:`~otcextensions.sdk.auto_scaling.v1.instance.Instance`
            instance
        :param action: Action to execute on instances:
            [``ADD``, ``REMOVE``, ``PROTECT``, ``UNPROTECT``]
        :param bool delete_instance: When set to ``True``, instance will be
            deleted after removed

        :returns: None
        """
        act = action.upper()
        if act not in self.ACTION_TYPES:
            msg = (_('Action type %(action)s is not supported %(types)s') %
                   {'action': action, 'types': self.ACTION_TYPES})
            raise exceptions.SDKException(msg)
        if delete_instance and act != 'REMOVE':
            msg = (_('Action type %s does not support delete_instance arg') %
                   (action))
            raise exceptions.SDKException(msg)
        ids = [instance.id if isinstance(instance, Instance) else instance
               for instance in instances]
        json_body = {
            'action': act,
            'instances_id': ids
        }
        if delete_instance:
            json_body['instance_delete'] = 'yes'
        return self._action(session, json_body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import uuid

import openstack
from openstack import _log

from otcextensions.tests.functional.sdk.mrs import TestMrs

_logger = _log.setup_logging('openstack')


class TestJobbinary(TestMrs):
    jobbinary = None

    def setUp(self):
        super(TestJobbinary, self).setUp()
        res = self.client.create_jobbinary(
            name=uuid.uuid4().hex,
            url="/simple/mapreduce/program",
            description='this is the job binary template',
        )
        id = res.id
        self.jobbinary = self.client.get_jobbinary(id)

        _logger.debug("SAHARA create test jobbinary")
        _logger.debug(self.jobbinary)

    def tearDown(self):
        try:
            pass
            if self.jobbinary.id:
                pass
                self.client.delete_jobbinary(self.jobbinary)
        except openstack.exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        super(TestJobbinary, self).tearDown()

    def test_list(self):
        self.jb = list(self.client.jobbinaries())
        self.assertGreaterEqual(len(self.jb), 0)
        for bin in self.jb:
            _logger.debug(bin)

    def test_update(self):
        res = self.client.update_jobbinary(
            self.jobbinary,
            is_protected=False,
            is_public=False,
            description='updated'
        )

        self.assertEqual('updated', res.description)
        _logger.debug(res)
        self.jbs = list(self.client.jobbinaries())
        self.assertGreaterEqual(len(self.jbs), 0)
        for jb in self.jbs:
            _logger.debug(jb)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DMS Queue v1 action implementations'''
import argparse
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ListDMSQueue(command.Lister):
    _description = _('List DMS Queues')
    columns = ('ID', 'name', 'queue_mode', 'description', 'redrive_policy',
               'max_consume_count', 'retention_hours')

    def get_parser(self, prog_name):
        parser = super(ListDMSQueue, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dms

        data = client.queues()

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns,
                 ) for s in data))
        return table


class ShowDMSQueue(command.ShowOne):
    _description = _('Show single Queue details')
    columns = ('ID', 'name', 'queue_mode', 'description', 'redrive_policy',
               'max_consume_count', 'retention_hours')

    def get_parser(self, prog_name):
        parser = super(ShowDMSQueue, self).get_parser(prog_name)
        parser.add_argument(
            'queue',
            metavar='<queue>',
            help=_('ID of the queue')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dms

        obj = client.get_queue(parsed_args.queue)

        # display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, self.columns)

        return (self.columns, data)


class DeleteDMSQueue(command.Command):
    _description = _('Delete DMS Queue')

    def get_parser(self, prog_name):
        parser = super(DeleteDMSQueue, self).get_parser(prog_name)
        parser.add_argument(
            'queue',
            metavar='<queue>',
            nargs='+',
            help=_('ID of the queue')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.queue:
            client = self.app.client_manager.dms
            for queue in parsed_args.queue:
                client.delete_queue(queue)


class CreateDMSQueue(command.ShowOne):
    _description = _('Create DMS Queue')
    columns = ('ID', 'name', 'queue_mode', 'description', 'redrive_policy',
               'max_consume_count', 'retention_hours')

    def get_parser(self, prog_name):
        parser = super(CreateDMSQueue, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the cluster.')
        )
        parser.add_argument(
            'queue_mode',
            metavar='<queue_mode>',
            help=_('Indicates the queue type.\n'
                   'Options:\n'
                   '* `NORMAL`: Standard queue. Best-effort ordering. '
                   'Messages might be retrieved in an order different from '
                   'which they were sent. Select standard queues when '
                   'throughput is important.\n'
                   '* `FIFO`: First-ln-First-out (FIFO) queue. FIFO delivery. '
                   'Messages are retrieved in the order they were sent. '
                   'Select FIFO queues when the order of messages is '
                   'important.\n'
                   '* `KAFKA_HA`: High-availability Kafka queue. All message '
                   'replicas are flushed to a disk synchronously. Select the '
                   'high availability mode when message reliability is '
                   'important.\n'
                   '* `KAFKA_HT`: High-throughput Kafka queue. All message '
                   'replicas are flushed to a disk asynchronously. Select the '
                   'high throughput mode when message delivery performance '
                   'is important.\n'
                   'Default value: `NORMAL`.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Indicates the basic information about a queue.\n'
                   'The queue description must be 0 to 160 characters in '
                   'length, and does not contain angle brackets (<) and (>).\n'
                   'NOTE: "\\" is defined as an escape character in the queue '
                   'description. If you need to enter a backward slash (\\) '
                   'or a double quotation mark (") in the queue description, '
                   'enter \\ or \\" ')
        )
        parser.add_argument(
            '--redrive_policy',
            metavar='<redrive_policy>',
            help=_('This parameter is mandatory only when queue_mode is '
                   'NORMAL or FIFO.\n'
                   'Indicates whether to enable dead letter messages. '
                   'Dead letter messages indicate messages that cannot be '
                   'normally consumed.\n'
                   'If a message fails to be consumed after the number of '
                   'consumption attempts of this message reaches the maximum '
                   'value, DMS stores this message into the dead letter queue.'
                   'This message will be retained in the deal letter queue '
                   'for 72 hours. During this period, consumers can consume '
                   'the dead letter message.\n'
                   'Dead letter messages can be consumed only by the consumer '
                   'group that generates these dead letter messages.\n'
                   'Dead letter messages of a FIFO queue are stored and '
                   'consumed based on the FIFO sequence.\n'
                   'Options:\n'
                   '* `enable`\n'
                   '* `disable`\n'
                   'Default value: disable.')
        )
        parser.add_argument(
            '--max_consume_count',
            metavar='<max_consume_count>',
            type=int,
            help=_('This parameter is mandatory only when redrive_policy '
                   'is set to enable.\n'
                   'This parameter indicates the maximum number of allowed '
                   'message consumption failures. When a message fails to be '
                   'consumed after the number of consumption attempts of '
                   'this message reaches this value, DMS stores this message '
                   'into the dead letter queue.\n'
                   'Value range: 1-100.')
        )
        parser.add_argument(
            '--retention_hours',
            metavar='<retention_hours>',
            type=int,
            help=_('This parameter is mandatory only when queue_mode is '
                   'set to KAFKA_HA or KAFKA_HT.\n'
                   'This parameter indicates the retention time of messages '
                   'in Kafka queues.\n'
                   'Value range: 1 to 72 hours.')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}

        attrs['name'] = parsed_args.name
        if parsed_args.queue_mode:
            mode = parsed_args.queue_mode.upper()
            if mode in ('NORMAL', 'FIFO', 'KAFKA_HA', 'KAFKA_HT'):
                attrs['queue_mode'] = mode
            else:
                msg = _('Queue mode is not in (NORMAL, FIFO, KAFKA_HA, '
                        'KAFKA_HT)')
                raise argparse.ArgumentTypeError(msg)
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.redrive_policy:
            attrs['redrive_policy'] = parsed_args.redrive_policy
        if parsed_args.retention_hours:
            attrs['retention_hours'] = parsed_args.retention_hours
        if parsed_args.max_consume_count:
            attrs['max_consume_count'] = parsed_args.max_consume_count

        client = self.app.client_manager.dms

        obj = client.create_queue(**attrs)

        data = utils.get_item_properties(obj, self.columns)

        return (self.columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils


class Instance(resource.Resource):

    resources_key = 'instances'

    base_path = '/instances'

    # capabilities
    allow_create = True
    allow_list = True
    allow_delete = True
    allow_fetch = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'limit', 'start', 'name', 'status',
        'includeFailure', 'exactMatchName')

    # Properties
    #: AZ where a cache node resides.
    #: The value of this parameter in the response contains an AZ ID.
    available_zones = resource.Body('available_zones', type=list)
    #: Backup policy.
    backup_policy = resource.Body('instance_backup_policy', type=dict)
    #:  Cache capacity in GB
    #: *Type: int*
    capacity = resource.Body('capacity', type=int)
    #: Billing mode.
    #: 0 indicates that users only pay for what they use.
    charging_mode = resource.Body('charging_mode')
    #: Time at which the DCS instance is created.
    #: For example, 2017-03-31T12:24:46.297Z.
    created_at = resource.Body('created_at')
    #: Instance description
    description = resource.Body('description')
    #: Domain name.
    domain_name = resource.Body('domainName')
    #: Instance engine
    engine = resource.Body('engine')
    #: Cache engine version.
    engine_version = resource.Body('engine_version')
    #: Error code returned when the DCS instance fails to be created
    #: or is in abnormal status.
    error_code = resource.Body('error_code')
    #: Instance Id
    id = resource.Body('instance_id', alternate_id=True)
    #: Internal DCS version.
    internal_version = resource.Body('internal_version')
    #: Cache node's IP address in tenant's VPC.
    ip = resource.Body('ip')
    #: Account lockout duration.
    lock_time = resource.Body('lock_time')
    #: Remaining time before the account is unlocked.
    lock_time_left = resource.Body('lock_time_left')
    #: Time at which the maintenance time window starts.
    #: Format: HH:mm:ss.
    maintain_begin = resource.Body('maintain_begin')
    #: Time at which the maintenance time window ends.
    #: Format: HH:mm:ss.
    maintain_end = resource.Body('maintain_end')
    #: Overall memory size.
    #: Unit: MB.
    max_memory = resource.Body('max_memory', type=int)
    #: Return message of the operation on the instance
    message = resource.Body('message')
    #: Instance name
    name = resource.Body('name')
    #: Order ID.
    #: An order ID is generated only in the monthly or yearly
    #: billing mode. In other billing modes, no value is returned
    #: for this parameter.
    order_id = resource.Body('order_id')
    #: Password.
    password = resource.Body('password')
    #: Port of the cache node.
    port = resource.Body('port')
    #: Product ID used to differentiate DCS instance types.
    #: * OTC_DCS_SINGLE: indicates a single-node DCS instance.
    #: * OTC_DCS_MS: indicates a master/standby DCS instance.
    #: * OTC_DCS_CL: indicates a DCS instance in cluster mode.
    product_id = resource.Body('product_id')
    #: Result of the operation on the instance
    result = resource.Body('result')
    #: Count of the change password retries
    #: *Type: str*
    retry_times_left = resource.Body('retry_times_left')
    security_group_id = resource.Body('security_group_id')
    security_group_name = resource.Body('security_group_name')
    #: Cache instance status. Instance Statuses.
    status = resource.Body('status')
    subnet_id = resource.Body('subnet_id')
    subnet_name = resource.Body('subnet_name')
    subnet_cidr = resource.Body('subnet_cidr')
    #: Resource specifications.
    #: * dcs.single_node: indicates a DCS instance in single-node mode.
    #: * dcs.master_standby: indicates a DCS instance in master/standby mode.
    #: * dcs.cluster: indicates a DCS instance in cluster mode.
    resource_spec_code = resource.Body('resource_spec_code')
    #: Size of the used memory.
    #: Unit: MB.
    used_memory = resource.Body('used_memory', type=int)
    user_id = resource.Body('user_id')
    user_name = resource.Body('user_name')
    vpc_id = resource.Body('vpc_id')
    vpc_name = resource.Body('vpc_name')

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~openstack.resource.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        session = cls._get_session(session)
        # Try to short-circuit by looking directly for a matching ID.
        try:
            match = cls.existing(
                id=name_or_id,
                connection=session._get_connection(),
                **params)
            return match.fetch(session, **params)

        # Ecosystems:
        # Add additional Exceptions to avoid abort of find method
        except (exceptions.NotFoundException, exceptions.HttpException,
                exceptions.MethodNotSupported, exceptions.BadRequestException):
            session.log.warn('Please specify instance ID if known for '
                             'better performance')

        if ('name' in cls._query_mapping._mapping.keys()
                and 'name' not in params):
            params['name'] = name_or_id

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))

    def change_pwd(self, session, current_password, new_password):
        """Schange instance password
        """
        body = {
            'old_password': current_password,
            'new_password': new_password
        }
        url = utils.urljoin(self.base_path, self.id, 'password')
        response = session.put(
            url,
            json=body)
        return self._translate_response(response, False)

    def extend(self, session, capacity):
        """Extend instance capacity
        """
        body = {'new_capacity': capacity}
        url = utils.urljoin(self.base_path, self.id, 'extend')
        response = session.post(
            url,
            json=body)
        return self._translate_response(response, False)

    def stop(self, session):
        """Stop instance
        """
        return self._status_action(session, 'stop')

    def start(self, session):
        """Start instance
        """
        return self._status_action(session, 'start')

    def restart(self, session):
        """Retart instance
        """
        return self._status_action(session, 'restart')

    def _status_action(self, session, action):
        '''perform status action on instance'''
        body = {'action': action, 'instances': [self.id]}
        url = utils.urljoin(self.base_path, 'status')
        response = session.put(
            url,
            json=body)
        return self._translate_response(response, False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource

from otcextensions.sdk.identity.v3 import _bad_base as _base


class Agency(_base.BadBaseResource):
    resource_key = 'agency'
    resources_key = 'agencies'
    base_path = '/v3.0/OS-AGENCY/agencies'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    commit_method = 'PUT'

    _query_mapping = resource.QueryParameters(
        'domain_id', 'name', 'trust_domain_id'
    )

    # Properties
    #: Time when an agency is created.
    created_at = resource.Body('create_time')
    #: Description of an agency.
    description = resource.Body('description')
    #: ID of the current domain.
    domain_id = resource.Body('domain_id')
    #: Validity period of an agency. The default value is null, indicating that
    #: the agency is permanently valid.
    duration = resource.Body('duration')
    #: Expiration time of an agency.
    expire_at = resource.Body('expire_time')
    #: ID of the delegated domain.
    trust_domain_id = resource.Body('trust_domain_id')
    #: Name of the delegated domain.
    trust_domain_name = resource.Body('trust_domain_name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.dcs.v1 import config

FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
FAKE_INSTANCE_ID = "some_fake_id"
EXAMPLE = {
    "description": "some description",
    "param_id": FAKE_ID,
    "param_name": "maxmemory-policy",
    "param_value": "noeviction",
    "default_value": "noeviction",
    "value_type": "Enum",
    "value_range": "volatile-lru,allkeys-lru,volatile-random,"
}


class TestConfig(base.TestCase):

    def setUp(self):
        super(TestConfig, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = config.Config()

        self.assertEqual('/instances/%(instance_id)s/configs', sot.base_path)

        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = config.Config(instance_id=FAKE_INSTANCE_ID, **EXAMPLE)
        self.assertEqual(FAKE_INSTANCE_ID, sot.instance_id)
        self.assertEqual(EXAMPLE['param_id'], sot.id)
        self.assertEqual(EXAMPLE['param_name'], sot.name)
        self.assertEqual(EXAMPLE['param_value'], sot.value)
        self.assertEqual(EXAMPLE['default_value'], sot.default_value)
        self.assertEqual(EXAMPLE['value_type'], sot.value_type)
        self.assertEqual(EXAMPLE['value_range'], sot.value_range)

    def test_construct_param_for_update(self):
        sot = config.Config(instance_id=FAKE_INSTANCE_ID)
        sot_dirty = {
            'param_name': 'name',
            'param_value': 'val',
            'param_id': 'id',
            'dummy': 'dummy'
        }
        sot_expected = {
            'param_name': 'name',
            'param_value': 'val',
            'param_id': 'id',
        }

        res = sot._construct_dict_for_update(sot_dirty)
        self.assertDictEqual(sot_expected, res)

    def test_update(self):

        sot = config.Config(instance_id=FAKE_INSTANCE_ID)
        sot2 = {
            'param_name': 'name',
            'param_value': 'val',
            'param_id': 'id',
        }

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {}
        mock_response.headers = {}

        self.sess.put.return_value = mock_response

        sot._update(
            self.sess, [sot2]
        )

        self.sess.put.assert_called_once_with(
            '/instances/%s/configs' % FAKE_INSTANCE_ID,
            json={
                'redis_config': [sot2]}
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from openstackclient.tests.unit import utils as tests_utils
from osc_lib import exceptions

from otcextensions.osclient.modelartsv2.v2 import dataset_sync
from otcextensions.tests.unit.osclient.modelartsv2.v2 import fakes


class TestDatasetSyncStatus(fakes.TestModelartsv2):
    _dataset = fakes.FakeDataset.create_one()

    columns = ("dataset_id", "status")
    object = fakes.FakeDatasetSync.create_one()

    data = fakes.gen_data(
        object,
        columns,
    )

    def setUp(self):
        super(TestDatasetSyncStatus, self).setUp()

        self.cmd = dataset_sync.DatasetSyncStatus(self.app, None)

        self.client.find_dataset = mock.Mock(return_value=self._dataset)
        self.client.get_dataset_sync_status = mock.Mock(
            return_value=self.object
        )

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [
            "dataset-id",
        ]

        verifylist = [
            ("dataset", "dataset-id"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.get_dataset_sync_status.assert_called_with(self._dataset)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            "nonexisting-dataset-id",
        ]

        verifylist = [
            ("dataset", "nonexisting-dataset-id"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        get_mock_result = exceptions.CommandError("Resource Not Found")
        self.client.find_dataset = mock.Mock(side_effect=get_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual("Resource Not Found", str(e))
        self.client.find_dataset.assert_called_with(
            "nonexisting-dataset-id", ignore_missing=False
        )


class TestDatasetSync(fakes.TestModelartsv2):
    _dataset = fakes.FakeDataset.create_one()

    def setUp(self):
        super(TestDatasetSync, self).setUp()

        self.client.find_dataset = mock.Mock(return_value=self._dataset)
        self.client.dataset_sync = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = dataset_sync.DatasetSync(self.app, None)

    def test_sync(self):
        arglist = [
            "dataset-id",
        ]

        verifylist = [
            ("dataset", "dataset-id"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.dataset_sync.assert_called_with(self._dataset)
        self.assertIsNone(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource
from otcextensions.sdk.modelartsv2.v2 import _base


class WorkerSpec(resource.Resource):
    #: Creation time.
    create_time = resource.Body("create_time", type=int)
    #: Labeling team member description.
    description = resource.Body("description")
    #: Email address of a labeling team member.
    email = resource.Body("email")
    #: Role.
    role = resource.Body("role", type=int)
    #: Current login status of a labeling team member.
    status = resource.Body("status", type=int)
    #: Update time.
    update_time = resource.Body("update_time", type=int)
    #: ID of a labeling team member.
    worker_id = resource.Body("worker_id")
    #: ID of a labeling team.
    workforce_id = resource.Body("workforce_id")


class WorkforceDescriptorSpec(resource.Resource):
    #: ID of a team labeling task.
    current_task_id = resource.Body("current_task_id")
    #: Name of a team labeling task.
    current_task_name = resource.Body("current_task_name")
    #: Whether to synchronously update auto labeling data.
    is_synchronize_auto_labeling_data = resource.Body(
        "is_synchronize_auto_labeling_data", type=bool
    )
    #: Whether to synchronize updated data, such as uploading files,
    #:  synchronizing data sources, and assigning imported unlabeled files to
    #:  team members.
    is_synchronize_data = resource.Body("is_synchronize_data", type=bool)
    #: Number of rejected samples.
    reject_num = resource.Body("reject_num", type=int)
    #: Number of persons who label each sample.
    repetition = resource.Body("repetition", type=int)
    #: List of labeling team members.
    workers = resource.Body("workers", type=list, list_type=WorkerSpec)
    #: ID of a labeling team.
    workforce_id = resource.Body("workforce_id")
    #: Name of a labeling team.
    workforce_name = resource.Body("workforce_name")


class LabelPropertySpec(resource.Resource):
    #: Default attribute: Label color, which is a hexadecimal code of the
    #:  color.
    color = resource.Body("@modelarts:color")
    #: Default attribute: Default shape of an object detection label
    #:  (dedicated attribute).
    default_shape = resource.Body("@modelarts:default_shape")
    #: Default attribute: Type of the head entity in the triplet relationship
    #:  label.
    from_type = resource.Body("@modelarts:from_type")
    #: Default attribute: The new name of the label.
    rename_to = resource.Body("@modelarts:rename_to")
    #: Default attribute: Label shortcut key.
    shortcut = resource.Body("@modelarts:shortcut")
    #: Default attribute: Type of the tail entity in the triplet relationship
    #:  label.
    to_type = resource.Body("@modelarts:to_type")


class LabelAttributeValueSpec(resource.Resource):
    #: Label attribute value ID.
    id = resource.Body("id")
    #: Label attribute value.
    value = resource.Body("value")


class LabelAttributeSpec(resource.Resource):
    #: Default value of a label attribute.
    default_value = resource.Body("default_value")
    #: Label attribute ID.
    id = resource.Body("id")
    #: Label attribute name.
    name = resource.Body("name")
    #: Label attribute type.
    type = resource.Body("type")
    #: List of label attribute values.
    values = resource.Body(
        "values", type=list, list_type=LabelAttributeValueSpec
    )


class LabelSpec(resource.Resource):
    #: Multi-dimensional attribute of a label.
    attributes = resource.Body(
        "attributes", type=list, list_type=LabelAttributeSpec
    )
    #: Label name.
    name = resource.Body("name")
    #: Basic attribute key-value pair of a label, such as color and shortcut
    #:  keys.
    property = resource.Body("property", type=LabelPropertySpec)
    #: Label type.
    type = resource.Body("type", type=int)


class LabelStatsSpec(resource.Resource):
    #: Multi-dimensional attribute of a label.
    attributes = resource.Body(
        "attributes", type=list, list_type=LabelAttributeSpec
    )
    #: Number of labels.
    count = resource.Body("count", type=int)
    #: Label name.
    name = resource.Body("name")
    #: Basic attribute key-value pair of a label, such as color and shortcut
    #:  keys.
    property = resource.Body("property", type=LabelPropertySpec)
    #: Number of samples containing the label.
    sample_count = resource.Body("sample_count", type=int)
    #: Label type.
    type = resource.Body("type", type=int)


class LabelFormatSpec(resource.Resource):
    #: Label type of text classification.
    label_type = resource.Body("label_type")
    #: Separator between labels.
    text_label_separator = resource.Body("text_label_separator")
    #: Separator between the text and label.
    text_sample_separator = resource.Body("text_sample_separator")


class DatasetVersionSpec(resource.Resource):
    #: Number of added samples.
    add_sample_count = resource.Body("add_sample_count", type=int)
    #: Number of samples with labeled versions.
    annotated_sample_count = resource.Body("annotated_sample_count", type=int)
    #: Number of labeled subsamples.
    annotated_sub_sample_count = resource.Body(
        "annotated_sub_sample_count", type=int
    )
    #: Whether to clear hard example properties during release.
    clear_hard_property = resource.Body("clear_hard_property", type=bool)
    #: Status code of a preprocessing task such as rotation and cropping.
    code = resource.Body("code")
    #: Time when a version is created.
    created_at = resource.Body("create_time", type=int)
    #: Whether to crop the image.
    crop = resource.Body("crop", type=bool)
    #: Path for storing cropped files.
    crop_path = resource.Body("crop_path")
    #: Temporary directory for executing the rotation and cropping task.
    crop_rotate_cache_path = resource.Body("crop_rotate_cache_path")
    #: Path for storing data.
    data_path = resource.Body("data_path")
    #: Sample statistics on a dataset, including the statistics on sample
    #:  metadata in JSON format.
    data_statistics = resource.Body("data_statistics", type=dict)
    #: Whether data is validated by the validation algorithm before release.
    data_validate = resource.Body("data_validate", type=bool)
    #: Number of deleted samples.
    deleted_sample_count = resource.Body("deleted_sample_count", type=int)
    #: Deletion reason statistics.
    deletion_stats = resource.Body("deletion_stats", type=dict)
    #: Description of a version.
    description = resource.Body("description")
    #: Whether to export images to the version output directory during
    #:  release.
    export_images = resource.Body("export_images", type=bool)
    #: Whether to parse the subsample number during release.
    extract_serial_number = resource.Body("extract_serial_number", type=bool)
    #: Whether to include the source data of a dataset during release.
    include_dataset_data = resource.Body("include_dataset_data", type=bool)
    #: Whether the current dataset version is used.
    is_current = resource.Body("is_current", type=bool)
    #: Label statistics list of a released version.
    label_stats = resource.Body(
        "label_stats", type=list, list_type=LabelStatsSpec
    )
    #: Label type of a released version.
    label_type = resource.Body("label_type")
    #: Input path for the manifest file cache during version release.
    manifest_cache_input_path = resource.Body("manifest_cache_input_path")
    #: Path for storing the manifest file with the released version.
    manifest_path = resource.Body("manifest_path")
    #: Task information recorded during release (for example, error
    #:  information).
    message = resource.Body("message")
    #: Number of modified samples.
    modified_sample_count = resource.Body("modified_sample_count", type=int)
    #: Number of labeled samples of parent versions.
    previous_annotated_sample_count = resource.Body(
        "previous_annotated_sample_count", type=int
    )
    #: Total samples of parent versions.
    previous_total_sample_count = resource.Body(
        "previous_total_sample_count", type=int
    )
    #: Parent version ID.
    previous_version_id = resource.Body("previous_version_id")
    #: ID of a preprocessing task such as rotation and cropping.
    processor_task_id = resource.Body("processor_task_id")
    #: Status of a preprocessing task such as rotation and cropping.
    processor_task_status = resource.Body("processor_task_status", type=int)
    #: Whether to clear the existing usage information of a dataset during
    #:  release.
    remove_sample_usage = resource.Body("remove_sample_usage", type=bool)
    #: Whether to rotate the image.
    rotate = resource.Body("rotate", type=bool)
    #: Path for storing the rotated file.
    rotate_path = resource.Body("rotate_path")
    #: Sample status.
    sample_state = resource.Body("sample_state")
    #: Status of a dataset version.
    status = resource.Body("status", type=int)
    #: Key identifier list of the dataset.
    tags = resource.Body("tags", type=list)
    #: Labeling task type of the released version, which is the same as the
    #:  dataset type.
    task_type = resource.Body("task_type", type=int)
    #: Total number of version samples.
    total_sample_count = resource.Body("total_sample_count", type=int)
    #: Total number of subsamples generated from the parent samples.
    total_sub_sample_count = resource.Body("total_sub_sample_count", type=int)
    #: Split training and verification ratio during version release.
    train_evaluate_sample_ratio = resource.Body("train_evaluate_sample_ratio")
    #: Time when a version is updated.
    update_time = resource.Body("update_time", type=int)
    #: Format of a dataset version.
    version_format = resource.Body("version_format")
    #: Dataset version ID.
    version_id = resource.Body("version_id")
    #: Dataset version name.
    version_name = resource.Body("version_name")
    #: Whether the first row in the released CSV file is a column name.
    with_column_header = resource.Body("with_column_header", type=bool)


class FieldSpec(resource.Resource):
    #: Schema description.
    description = resource.Body("description")
    #: Schema name.
    name = resource.Body("name")
    #: Schema ID.
    schema_id = resource.Body("schema_id", type=int)
    #: Schema value type.
    type = resource.Body("type")


class SourceInfoSpec(resource.Resource):
    #: ID of an MRS cluster.
    cluster_id = resource.Body("cluster_id")
    #: Running mode of an MRS cluster.
    cluster_mode = resource.Body("cluster_mode")
    #: Name of an MRS cluster.
    cluster_name = resource.Body("cluster_name")
    #: Name of the database to which the table dataset is imported.
    database_name = resource.Body("database_name")
    #: HDFS path of a table dataset.
    input = resource.Body("input")
    #: IP address of your GaussDB(DWS) cluster.
    ip = resource.Body("ip")
    #: Port number of your GaussDB(DWS) cluster.
    port = resource.Body("port")
    #: DLI queue name of a table dataset.
    queue_name = resource.Body("queue_name")
    #: Subnet ID of an MRS cluster.
    subnet_id = resource.Body("subnet_id")
    #: Name of the table to which a table dataset is imported.
    table_name = resource.Body("table_name")
    #: Username, which is mandatory for GaussDB(DWS) data.
    user_name = resource.Body("user_name")
    #: User password, which is mandatory for GaussDB(DWS) data.
    user_password = resource.Body("user_password")
    #: ID of the VPC where an MRS cluster resides.
    vpc_id = resource.Body("vpc_id")


class SchemaMapSpec(resource.Resource):
    #: Name of the destination column.
    dest_name = resource.Body("dest_name")
    #: Name of the source column.
    src_name = resource.Body("src_name")


class DataSourceSpec(resource.Resource):
    #: Data source path.
    data_path = resource.Body("data_path")
    #: Data type.
    data_type = resource.Body("data_type", type=int)
    #: Schema mapping information corresponding to the table data.
    schema_maps = resource.Body(
        "schema_maps", type=list, list_type=SchemaMapSpec
    )
    #: Information required for importing a table data source.
    source_info = resource.Body("source_info", type=SourceInfoSpec)
    #: Whether the first row in the file is a column name.
    with_column_header = resource.Body("with_column_header", type=bool)


class Dataset(_base.Resource):
    base_path = "/datasets"
    resources_count_key = "total_number"
    resources_key = "datasets"

    # capabilities
    allow_create = True
    allow_list = True
    allow_delete = True
    allow_fetch = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        "check_running_task",
        "contain_versions",
        "dataset_type",
        "file_preview",
        "limit",
        "offset",
        "order",
        "running_task_type",
        "search_content",
        "sort_by",
        "support_export",
        "train_evaluate_ratio",
        "version_format",
        "with_labels",
        "workspace_id",
    )

    # Properties
    #: AI Project
    ai_project = resource.Body("ai_project")
    #: Number of labeled samples in a dataset.
    annotated_sample_count = resource.Body("annotated_sample_count", type=int)
    #: Number of labeled subsamples.
    annotated_sub_sample_count = resource.Body(
        "annotated_sub_sample_count", type=int
    )
    #: Whether to enable content labeling for the speech paragraph labeling
    #:  dataset.
    content_labeling = resource.Body("content_labeling", type=bool)
    #: Time when a dataset is created.
    create_time = resource.Body("create_time", type=int)
    #: Current version ID of a dataset.
    current_version_id = resource.Body("current_version_id")
    #: Current version name of a dataset.
    current_version_name = resource.Body("current_version_name")
    #: Data format.
    data_format = resource.Body("data_format")
    #: Data source list.
    data_sources = resource.Body(
        "data_sources", type=list, list_type=DataSourceSpec
    )
    #: Sample statistics on a dataset, including the statistics on sample
    #:  metadata.
    data_statistics = resource.Body("data_statistics", type=dict)
    #: Time when a sample and a label are updated.
    data_update_time = resource.Body("data_update_time", type=int)
    #: Dataset format.
    dataset_format = resource.Body("dataset_format", type=int)
    #: Dataset ID.
    dataset_id = resource.Body("dataset_id", alternate_id=True)
    #: Dataset name.
    dataset_name = resource.Body("dataset_name")
    #: Key identifier list of a dataset, for example, ["Image","Object
    #:  detection"].
    dataset_tags = resource.Body("dataset_tags", type=list)
    #: Dataset type.
    dataset_type = resource.Body("dataset_type", type=int)
    #: Dataset version.
    dataset_version = resource.Body("dataset_version")
    #: Number of dataset versions.
    dataset_version_count = resource.Body("dataset_version_count", type=int)
    #: Number of deleted samples.
    deleted_sample_count = resource.Body("deleted_sample_count", type=int)
    #: Deletion reason statistics.
    deletion_stats = resource.Body("deletion_stats", type=dict)
    #: Dataset description.
    description = resource.Body("description")
    #: Enterprise project ID.
    enterprise_project_id = resource.Body("enterprise_project_id")
    #: Whether the dataset contains running (including initialization) tasks.
    exist_running_task = resource.Body("exist_running_task", type=bool)
    #: Whether the dataset contains team labeling tasks.
    exist_workforce_task = resource.Body("exist_workforce_task", type=bool)
    #: List of features supported by the dataset.
    feature_supports = resource.Body("feature_supports", type=list)
    #: Whether to automatically import the labeling information in the input
    #:  directory, supporting detection, image classification, and text
    #:  classification.
    import_annotations = resource.Body("import_annotations", type=bool)
    #: Whether to import data.
    import_data = resource.Body("import_data", type=bool)
    #: ID of an import task.
    import_task_id = resource.Body("import_task_id")
    #: Path for storing the labeling result of a dataset.
    inner_annotation_path = resource.Body("inner_annotation_path")
    #: Path for storing the internal data of a dataset.
    inner_data_path = resource.Body("inner_data_path")
    #: Path for storing internal logs of a dataset.
    inner_log_path = resource.Body("inner_log_path")
    #: Path for internal task of a dataset.
    inner_task_path = resource.Body("inner_task_path")
    #: Path for storing internal temporary files of a dataset.
    inner_temp_path = resource.Body("inner_temp_path")
    #: Output directory of a dataset.
    inner_work_path = resource.Body("inner_work_path")
    #: Label format information.
    label_format = resource.Body("label_format", type=LabelFormatSpec)
    #: Number of labeling tasks.
    label_task_count = resource.Body("label_task_count", type=int)
    #: Dataset label list.
    labels = resource.Body("labels", type=list, list_type=LabelSpec)
    #: Number of loading samples.
    loading_sample_count = resource.Body("loading_sample_count", type=int)
    #: Whether a dataset is hosted.
    managed = resource.Body("managed", type=bool)
    #: Dataset name.
    name = resource.Body("name", alias="dataset_name")
    #: Number of next versions of a dataset.
    next_version_num = resource.Body("next_version_num", type=int)
    #: ID list of running (including initialization) tasks.
    running_tasks_id = resource.Body("running_tasks_id", type=list)
    #: Schema list.
    schema = resource.Body("schema", type=list, list_type=FieldSpec)
    #: Dataset status.
    status = resource.Body("status", type=int)
    #: Third-party path.
    third_path = resource.Body("third_path")
    #: Total number of dataset samples.
    total_sample_count = resource.Body("total_sample_count", type=int)
    #: Total number of subsamples generated from the parent samples.
    total_sub_sample_count = resource.Body("total_sub_sample_count", type=int)
    #: Number of auto labeling samples to be confirmed.
    unconfirmed_sample_count = resource.Body(
        "unconfirmed_sample_count", type=int
    )
    #: Time when a dataset is updated.
    update_time = resource.Body("update_time", type=int)
    #: Dataset version information.
    versions = resource.Body(
        "versions", type=list, list_type=DatasetVersionSpec
    )
    #: Output dataset path, which is used to store output files such as label
    #:  files.
    work_path = resource.Body("work_path")
    #: Type of the dataset output path.
    work_path_type = resource.Body("work_path_type", type=int)
    #: Team labeling information.
    workforce_descriptor = resource.Body(
        "workforce_descriptor", type=WorkforceDescriptorSpec
    )
    #: Number of team labeling tasks of a dataset.
    workforce_task_count = resource.Body("workforce_task_count", type=int)
    #: Workspace ID.
    workspace_id = resource.Body("workspace_id")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.sdk.cts.v1 import tracker as _tracker
from otcextensions.sdk.cts.v1 import trace as _trace


class Proxy(proxy.Proxy):

    skip_discovery = True

    def traces(self, tracker='system', **query):
        """List all traces

        :param tracker: The name or an tracker of
            :class:`~otcextensions.sdk.cts.v1.tracker.Tracker`
        :param limit: default limit of resources
        :returns: A generator of tracker object of
            :class:`~otcextensions.sdk.cts.v1.trace.Trace`
        """
        if isinstance(tracker, str):
            tracker = self._get_resource(_tracker.Tracker, {'name': tracker})
        return self._list(_trace.Trace,
                          paginated=False,
                          tracker_name=tracker.name, **query)

    def get_tracker(self, tracker):
        """Get detail about a given tracker

        :param tracker: The tracker id, name or an tracker of
            :class:`~otcextensions.sdk.cts.v1.tracker.Tracker`
        :returns: one object of class
            :class:`~otcextensions.sdk.dcs.v1.tracker.Tracker`
        """
        return self._get(_tracker.Tracker, tracker, requires_id=False)

    def create_tracker(self, **kwargs):
        """Create a tracker

        :param dict kwargs: Keyword arguments which will be used to overwrite a
            :class:`~otcextensions.sdk.dcs.v1.tracker.Tracker`
        """
        return self._create(_tracker.Tracker, **kwargs)

    def update_tracker(self, tracker, **attrs):
        """Update tracker with attributes

        :param tracker: The value can be the ID of an tracker
            or a :class:`~otcextensions.sdk.dcs.v1.tracker.tracker`
            tracker.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dcs.v1.tracker.tracker`,
            comprised of the properties on the tracker class.
        :returns: The updated tracker
        :rtype: :class:`~otcextensions.sdk.dcs.v1.tracker.tracker`
        """
        return self._update(_tracker.Tracker, tracker, **attrs)

    def delete_tracker(self, tracker='system', ignore_missing=True):
        """Delete a tracker

        :param tracker: The value can be the ID of a tracker or a
            :class:`~otcextensions.sdk.cts.v2.tracker.Tracker`
            instance
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the backup_policy does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent backup_policy.

        :returns: tracker been deleted
        :rtype:
            :class:`~otcextensions.sdk.cts.v2.tracker.Tracker`
        """
        tracker = self._get_resource(_tracker.Tracker, tracker)
        return tracker._delete_tracker(self, tracker.id)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts training job v1 action implementations"""
import logging

from cliff import columns as cliff_columns
from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


_formatters = {
    "created_at": cli_utils.UnixTimestampFormatter,
    "config": cli_utils.YamlFormat,
}


def _get_columns(item):
    column_map = {}
    hidden = ["location"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class JobStatus(cliff_columns.FormattableColumn):
    CHOICES_MAP = {
        0: "UNKNOWN",
        1: "INIT",
        2: "IMAGE_CREATING",
        3: "IMAGE_FAILED",
        4: "SUBMIT_TRYING",
        5: "SUBMIT_FAILED",
        6: "DELETE_FAILED",
        7: "WAITING",
        8: "RUNNING",
        9: "KILLING",
        10: "COMPLETED",
        11: "FAILED",
        12: "KILLED",
        13: "CANCELED",
        14: "LOST",
        15: "SCALING",
        16: "SUBMIT_MODEL_FAILED",
        17: "DEPLOY_SERVICE_FAILED",
        18: "CHECK_INIT",
        19: "CHECK_RUNNING",
        20: "RUNNING_COMPLETED",
        21: "CHECK_FAILED",
    }
    STR = "\n".join(f"{key}: {value}" for key, value in CHOICES_MAP.items())

    def human_readable(self):
        return self.CHOICES_MAP.get(self._value, str(self._value))


class ListTrainingJobVersions(command.Lister):
    _description = _("Get the list of training job versions")
    columns = (
        "Version Id",
        "Version Name",
        "Status",
        "Created At",
        "Started At",
        "Duration",
    )

    def get_parser(self, prog_name):
        parser = super(ListTrainingJobVersions, self).get_parser(prog_name)
        parser.add_argument(
            "jobId",
            metavar="<jobId>",
            help=_("ID of the training job."),
        )
        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            help=_(
                "Number of jobs displayed on each page. The value "
                "range is [1, 1000]. Default value: 10."
            ),
        )
        parser.add_argument(
            "--offset",
            metavar="<offset>",
            type=int,
            help=_("Index of the page to be queried. Default value: 1"),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        params = {}
        if parsed_args.limit:
            params["limit"] = parsed_args.limit
        if parsed_args.offset:
            params["offset"] = parsed_args.offset

        data = client.training_job_versions(parsed_args.jobId, **params)

        _formatters = {
            "Created At": cli_utils.UnixTimestampFormatter,
            "Started At": cli_utils.UnixTimestampFormatter,
            "Status": JobStatus,
        }
        return (
            self.columns,
            (
                utils.get_dict_properties(
                    s, self.columns, formatters=_formatters
                )
                for s in data
            ),
        )


class CreateTrainingJobVersion(command.ShowOne):
    _description = _("Create a ModelArts Training Job Version")

    def get_parser(self, prog_name):
        parser = super(CreateTrainingJobVersion, self).get_parser(prog_name)
        parser.add_argument(
            "jobId",
            metavar="<jobId>",
            help=_("Training job ID."),
        )
        mandatary_group = parser.add_argument_group("mandatary arguments")
        parser._action_groups.insert(1, parser._action_groups.pop())
        mandatary_group.add_argument(
            "--worker-server-num",
            metavar="<worker_server_num>",
            type=int,
            required=True,
            help=_("Number of workers in a training job."),
        )
        mandatary_group.add_argument(
            "--app-url",
            metavar="<app_url>",
            required=True,
            help=_(
                "Code directory of a training job, for example, /usr/app/."
            ),
        )
        mandatary_group.add_argument(
            "--boot-file-url",
            metavar="<boot_file_url>",
            required=True,
            help=_(
                "Boot file of a training job, which needs to be stored "
                "in the code directory."
            ),
        )
        mandatary_group.add_argument(
            "--spec-id",
            metavar="<spec_id>",
            required=True,
            type=int,
            help=_(
                "ID of the resource specifications selected for a "
                "training job."
            ),
        )
        mandatary_group.add_argument(
            "--engine-id",
            metavar="<engine_id>",
            required=True,
            type=int,
            help=_(
                "ID of the engine selected for a training job. "
                "The default value is 1."
            ),
        )
        mandatary_group.add_argument(
            "--model-id",
            metavar="<model_id>",
            required=True,
            type=int,
            help=_("ID of the built-in model of a training job."),
        )
        mandatary_group.add_argument(
            "--pre-version-id",
            metavar="<pre_version_id>",
            required=True,
            type=int,
            help=_("ID of the previous version of a training job."),
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            dest="job_desc",
            help=_("Description of a training job."),
        )
        parser.add_argument(
            "--log-url",
            metavar="<log_url>",
            help=_(
                "Boot file of a training job, which needs to be stored "
                "in the code directory."
            ),
        )
        parser.add_argument(
            "--data-url",
            metavar="<data_url>",
            help=_("OBS URL of the dataset required by a training job."),
        )
        parser.add_argument(
            "--dataset-id",
            metavar="<dataset_id>",
            help=_("Dataset ID of a training job."),
        )
        parser.add_argument(
            "--dataset-version-id",
            metavar="<dataset_version_id>",
            help=_(
                "Dataset version ID of a training job. This parameter "
                "must be used together with dataset_id, but cannot be "
                "used together with data_url or data_source."
            ),
        )
        parser.add_argument(
            "--train-url",
            metavar="<train_url>",
            help=_("OBS URL of the output file of a training job."),
        )
        parser.add_argument(
            "--user-image-url",
            metavar="<user_image_url>",
            help=_("SWR URL of a custom image used by a training job."),
        )
        parser.add_argument(
            "--user-command",
            metavar="<user_command>",
            help=_(
                "Boot command used to start the container of a custom image of"
                "a training job. The format is bash /home/work/run_train.sh."
            ),
        )
        parser.add_argument(
            "--parameter",
            metavar="label=<label>,value=<value>",
            required_keys=["label", "value"],
            action=parseractions.MultiKeyValueAction,
            help=_(
                "Running parameters of a training job. It is a collection of "
                "label-value pairs\n."
                "Example: --parameter label=flower,value=rose "
                "--parameter label=color,value=red"
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        attrs = {"config": {}}
        if parsed_args.job_desc:
            attrs["job_desc"] = parsed_args.job_desc

        config_attrs = (
            "worker_server_num",
            "app_url",
            "boot_file_url",
            "parameter",
            "pre_version_id",
            "data_url",
            "dataset_id",
            "dataset_version_id",
            "spec_id",
            "engine_id",
            "model_id",
            "train_url",
            "log_url",
            "user_image_url",
            "user_command",
        )

        for config_attr in config_attrs:
            val = getattr(parsed_args, config_attr)
            if val:
                attrs["config"][config_attr] = val

        data = client.create_training_job_version(parsed_args.jobId, **attrs)

        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns, formatters=_formatters)

        return display_columns, data


class ShowTrainingJobVersion(command.ShowOne):
    _description = _("Show details of a modelarts Training Job Version")

    def get_parser(self, prog_name):
        parser = super(ShowTrainingJobVersion, self).get_parser(prog_name)
        parser.add_argument(
            "jobId",
            metavar="<jobId>",
            help=_("ID of a training job"),
        )
        parser.add_argument(
            "versionId",
            metavar="<versionId>",
            help=_("Version ID of a training job."),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        obj = client.get_training_job_version(
            parsed_args.jobId, parsed_args.versionId
        )
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return display_columns, data


class StopTrainingJobVersion(command.ShowOne):
    _description = _("Stop Training Job Version.")

    def get_parser(self, prog_name):
        parser = super(StopTrainingJobVersion, self).get_parser(prog_name)

        parser.add_argument(
            "jobId",
            metavar="<jobId>",
            help=_("ID of a training job"),
        )
        parser.add_argument(
            "versionId",
            metavar="<versionId>",
            help=_("ID of a training job version"),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        obj = client.stop_training_job_version(
            parsed_args.jobId, parsed_args.versionId
        )
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)
        return (display_columns, data)


class DeleteTrainingJobVersion(command.Command):
    _description = _("Delete ModelArts training job version.")

    def get_parser(self, prog_name):
        parser = super(DeleteTrainingJobVersion, self).get_parser(prog_name)
        parser.add_argument(
            "jobId",
            metavar="<jobId>",
            help=_("ID of the training job."),
        )
        parser.add_argument(
            "versionId",
            metavar="<versionId>",
            help=_("Version id of the training job."),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        client.delete_training_job_version(
            parsed_args.jobId, parsed_args.versionId
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils


class Cluster(resource.Resource):
    base_path = '/clusters'

    resource_key = 'cluster'
    resources_key = 'clusters'

    # capabilities
    allow_create = True
    allow_list = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True
    allow_patch = True

    # Properties
    #: The key indicates an ongoing task.
    action_progress = resource.Body('action_progress', type=dict)
    #: Availabilitu Zone of a Cluster
    availability_zone = resource.Body('availability_zone')
    #: Cluster creation time.
    created_at = resource.Body('created')
    #: Private network connection information about the cluster.
    endpoints = resource.Body('endpoints', type=list, list_type=dict)
    #: Enterprise project ID.
    #: If no enterprise project is specified for a cluster,
    #: the default enterprise project ID 0 is used.
    enterprise_project_id = resource.Body('enterprise_project_id')
    #: Cause of failure. If the parameter is left empty,
    #:  the cluster is in the normal state.
    error = resource.Body('failed_reasons', type=dict)
    #: Node Type (DWS Flavor).
    flavor = resource.Body('node_type')
    #: Node Type ID (DWS Flavor ID).
    flavor_id = resource.Body('node_type_id')
    #: Floating IP details
    floating_ip = resource.Body('public_ip', type=dict)
    #: Guest Agent Version
    guest_agent_version = resource.Body('guest_agent_version')
    #: Whether logical_cluster has been enabled.
    is_logical_cluster_enabled = resource.Body(
        'use_logical_cluster', type=bool
    )
    #: Whether logical_cluster has been initialed
    is_logical_cluster_initialed = resource.Body(
        'logical_cluster_initialed', type=bool
    )
    #: Whether logical_cluster_mode is set ``true``.
    is_logical_cluster_mode = resource.Body('logical_cluster_mode', type=bool)
    #: The number of latest manual snapshots that
    #:  need to be retained for a cluster.
    keep_last_manual_snapshot = resource.Body(
        'keep_last_manual_snapshot', type=int
    )
    #: Cluster maintenance window.
    maintenance_window = resource.Body('maintain_window', type=dict)
    #: Network ID, which is used for configuring cluster network.
    network_id = resource.Body('subnet_id')
    #: Nodes List.
    nodes = resource.Body('nodes', type=list)
    #: Number of deployed CNs.
    num_cn = resource.Body('number_of_cn', type=int)
    #: Number of events.
    num_recent_events = resource.Body('recent_event', type=int)
    #: number_of_free_node
    num_free_nodes = resource.Body('number_of_free_node', type=int)
    #: Number of nodes in a cluster. The value ranges from 3 to 32.
    num_nodes = resource.Body('number_of_node')
    #: Parameter group details.
    parameter_group = resource.Body('parameter_group', type=dict)
    #: plugins
    plugins = resource.Body('plugins', type=list)
    #: Service port of a cluster. The value ranges from 8000 to 30000.
    #: The default value is 8000.
    port = resource.Body('port')
    #: List of private network IP addresses.
    private_ip = resource.Body('private_ip', type=list)
    #: Public network connection information about the cluster.
    public_endpoints = resource.Body(
        'public_endpoints', type=list, list_type=dict
    )
    #: Cluster scale-out details.
    resize_info = resource.Body('resize_info', type=dict)
    #: Router ID, which is used for configuring cluster network.
    router_id = resource.Body('vpc_id')
    #: Cluster status. The value can be one of the following:
    #:  - CREATING
    #:  - AVAILABLE
    #:  - UNAVAILABLE
    #:  - CREATION FAILED
    status = resource.Body('status')
    #: Sub-status of clusters in the AVAILABLE state.
    sub_status = resource.Body('sub_status')
    #: spec_version
    spec_version = resource.Body('spec_version')
    #: ID of a security group, which is used for configuring cluster network
    security_group_id = resource.Body('security_group_id')
    #: Cluster management task.
    task_status = resource.Body('task_status')
    #: Labels in a cluster.
    tags = resource.Body('tags', type=list, list_type=dict)
    #: Administrator username for logging in to a GaussDB(DWS) cluster.
    user_name = resource.Body('user_name')
    #: Administrator password for logging in to a GaussDB(DWS) cluster.
    user_pwd = resource.Body('user_pwd')
    #: Last modification time of a cluster.
    updated_at = resource.Body('updated')
    #: Data warehouse version.
    version = resource.Body('version')

    # Computed Resources
    #: Private Domain from endpoints connection_info
    private_domain = resource.Computed('private_domain', default='')
    #: Public Domain from public_endpoints connection_info
    public_domain = resource.Computed('public_domain', default='')

    def delete(
        self, session, keep_last_manual_snapshot=0, ignore_missing=False
    ):
        """Delete a DWS Cluster.

        This function overrides default Resource.delete to enable params

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param int keep_last_manual_snapshot: The number of latest manual
            snapshots that need to be retained for a cluster.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the cluster does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent cluster.
        :returns: None
        """
        body = {'keep_last_manual_snapshot': keep_last_manual_snapshot}
        request = self._prepare_request()
        response = session.delete(request.url, json=body)
        try:
            self._translate_response(response, has_body=True)
        except exceptions.ResourceNotFound:
            if ignore_missing:
                return None
            raise

    def _action(self, session, action, body=None):
        """Preform actions given the message body."""
        uri = utils.urljoin('clusters', self.id, action)
        response = session.post(uri, json=body)
        exceptions.raise_from_response(response)

    def restart(self, session):
        """Restart the cluster."""
        self._action(
            session,
            'restart',
            {
                'restart': {},
            },
        )

    def scale_out(self, session, node_count):
        """Scale Out cluster Nodes."""
        self._action(
            session,
            'resize',
            {
                'scale_out': {
                    'count': node_count,
                }
            },
        )

    def reset_password(self, session, new_password):
        """Reset Admin DB Password."""
        self._action(
            session,
            'reset-password',
            {
                'new_password': new_password,
            },
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.cce.v3 import cluster_node

EXAMPLE_LIST = {
    "kind": "List",
    "apiVersion": "v3",
    "items": [
        {
            "kind": "Node",
            "apiVersion": "v3",
            "metadata": {
                "name": "node-demo",
                "uid": "c33b6898-38c9-11e9-b246-0255ac101413",
                "creationTimestamp": "2019-02-25 06:19:05.789461541 +0000 UTC",
                "updateTimestamp": "2019-02-25 06:19:05.789462592 +0000 UTC",
                "annotations": {
                    "kubernetes.io/node-pool.id":
                        "eu-de-02#s2.large.1#EulerOS 2.2"
                }
            },
            "spec": {
                "flavor": "s2.large.1",
                "az": "eu-de-02",
                "os": "EulerOS 2.2",
                "login": {
                    "sshKey": "KeyPair-demo",
                },
                "rootVolume": {
                    "volumetype": "SATA",
                    "size": 40
                },
                "dataVolumes": [
                    {
                        "volumetype": "SATA",
                        "size": 100
                    }
                ],
                "publicIP": {
                    "eip": {
                        "bandwidth": {}
                    }
                },
                "billingMode": 0
            },
            "status": {
                "phase": "Active",
                "serverId": "99de97f0-a10a-4215-ace7-817de0136ff5",
                "privateIP": "192.168.0.218",
                "publicIP": "10.154.50.127"
            }
        }
    ]
}

EXAMPLE_CREATE = {
    "kind": "Node",
    "apiVersion": "v3",
    "metadata": {
        "name": "mycluster-demo",
        "labels": {
            "foo": "bar"
        },
        "annotations": {
            "annotation1": "abc"
        }
    },
    "spec": {
        "flavor": "s1.large",
        "az": "eu-de",
        "login": {
            "sshKey": "KeyPair-demo"
        },
        "rootVolume": {
            "size": 40,
            "volumetype": "SATA"
        },
        "dataVolumes": [
            {
                "size": 100,
                "volumetype": "SATA"
            }
        ],
        "publicIP": {
            "count": 1,
            "eip": {
                "type": "5_bgp",
                "bandwidth": {
                    "chargemode": "traffic",
                    "size": 10,
                    "sharetype": "PER"
                }
            }
        },
        "count": 1
    }
}


class TestClusterNode(base.TestCase):

    def setUp(self):
        super(TestClusterNode, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

        self.sot = cluster_node.ClusterNode(cluster_id='cluster_id')

    def test_basic(self):
        sot = cluster_node.ClusterNode()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual(None, sot.resources_key)
        self.assertEqual('/clusters/%(cluster_id)s/nodes',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        obj = EXAMPLE_LIST['items'][0]
        sot = cluster_node.ClusterNode.existing(**obj)
        self.assertEqual(obj['metadata']['uid'], sot.id)
        self.assertEqual(obj['kind'], sot.kind)

    def test_list(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = copy.deepcopy(EXAMPLE_LIST)

        self.sess.get.return_value = mock_response

        result = list(
            self.sot.list(
                self.sess,
                cluster_id='cluster_id',
            )
        )

        self.sess.get.assert_called_once_with(
            '/clusters/%s/nodes' % self.sot.cluster_id,
            params={},
            headers={
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
        )

        expected_list = [
            cluster_node.ClusterNode.existing(
                **EXAMPLE_LIST['items'][0]),
        ]

        self.assertEqual(
            set(i.id for i in expected_list),
            set(i.id for i in result))



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Update CTS Tracker by using id or an instance of class Tracker
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


tracker = "system"
attrs = {
    "bucket_name": "cloudtraceservice",
    "file_prefix_name": "newPrefix-",
    "lts": {
        "is_lts_enabled": True,
        "log_group_name": "CTS",
        "log_topic_name": "system-trace",
        "log_group_id": "1186622b-78ec-11ea-997c-286ed488c87f",
        "log_topic_id": "751f0409-78ec-11ea-90c7-286ed488c880"
    },
    "status": "enabled",
    "tracker_name": "system",
    "detail": ""
}

tracker = conn.cts.get_tracker(tracker)
conn.cts.update_tracker(tracker, **attrs)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""Configuration v1 action implementations"""
import json
import logging

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.i18n import _


LOG = logging.getLogger(__name__)

DATASTORE_TYPE_CHOICES = ['MySQL', 'PostgreSQL', 'SQLServer']


def format_dict(data):
    """Return a formatted string of key value pairs

    :param data: a dict
    :rtype: a string formatted to key='value'
    """

    if data is None:
        return None

    output = ""
    for s in sorted(data):
        output = output + s + "='" + str(data[s]) + "',\n "
    return output[:-2]


class ListConfigurations(command.Lister):
    _description = _("List Parameter Groups")
    columns = ['ID', 'Name', 'Description', 'Datastore Name',
               'Datastore Version Name']

    def get_parser(self, prog_name):
        parser = super(ListConfigurations, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            dest='limit',
            metavar='<limit>',
            type=int,
            default=None,
            help=_('Limit the number of results displayed. (Not supported)')
        )
        parser.add_argument(
            '--marker',
            dest='marker',
            metavar='<ID>',
            help=_('Begin displaying the results for IDs greater than the '
                   'specified marker. When used with --limit, set this to '
                   'the last ID displayed in the previous run. '
                   '(Not supported)')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.configurations()

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )


class ShowConfiguration(command.ShowOne):
    _description = _("Shows details of a database configuration group.")
    columns = ['ID', 'Name', 'Description', 'Datastore Name',
               'Datastore Version Name', 'Values']

    def get_parser(self, prog_name):
        parser = super(ShowConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            'configuration_group',
            metavar="<configuration_group>",
            help=_("ID or name of the configuration group")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        obj = client.find_configuration(parsed_args.configuration_group)
        # TODO(agoncharov) find by name does not return parameter values

        # TODO(agoncharov) values and parameters are breaking the layout
        # dramatically. Maybe it make sence to create additional
        # filter to get/list only specific values/params
        data = utils.get_item_properties(
            obj, self.columns,
            formatters={
                'values': format_dict,
                'parameters': utils.format_list_of_dicts,
            }
        )

        return (self.columns, data)


class CreateConfiguration(command.ShowOne):
    _description = _("Create new Parameter Group")

    columns = (
        'id',
        'name',
        'description',
        'datastore_version_id',
        'datastore_version_name',
        'datastore_name',
        'created',
        'updated',
        'allowed_updated',
        'instance_count',
        'values'
    )

    def get_parser(self, prog_name):
        parser = super(CreateConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar="<name>",
            required=True,
            help=_("Parameter group name")
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help=_("Parameter group description")
        )
        parser.add_argument(
            '--datastore_type',
            metavar="<datastore_type>",
            choices=DATASTORE_TYPE_CHOICES,
            required=True,
            help=_("Datastore type")
        )
        parser.add_argument(
            '--datastore_version',
            metavar="<datastore_version>",
            required=True,
            help=_("Datastore version")
        )
        parser.add_argument(
            '--value',
            dest="ind_values",
            metavar="<key=value>",
            action=parseractions.KeyValueAction,
            help=_("Parameter group value"
                   "(repeat option to set multiple values)")
        )
        parser.add_argument(
            'values',
            metavar='<values>',
            help=_('Dictionary (JSon) of the values to set.'),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        config_attrs = {}
        config_attrs['datastore'] = {}
        if parsed_args.name:
            config_attrs['name'] = parsed_args.name
        if parsed_args.description:
            config_attrs['description'] = parsed_args.description
        if parsed_args.datastore_type and parsed_args.datastore_version:
            datastore = {}
            datastore['type'] = parsed_args.datastore_type
            datastore['version'] = parsed_args.datastore_version
            config_attrs['datastore'] = datastore

        # flatten values into the proper config_attrs
        values = {}
        try:
            values = json.loads(parsed_args.values)
        except Exception as e:
            msg = (_("Failed to parse configuration values: %(e)s")
                   % {'e': e})
            raise exceptions.CommandError(msg)

        if getattr(parsed_args, 'ind_values', None):
            for k, v in parsed_args.ind_values.items():
                values[k] = str(v)

        config_attrs['values'] = values

        config = client.create_configuration(**config_attrs)

        data = utils.get_item_properties(
            config, self.columns,
            formatters={
                'values': format_dict,
            }
        )

        return (self.columns, data)


class SetConfiguration(command.ShowOne):
    _description = _("Set values of the Parameter Group")

    columns = (
        'id',
        'name',
        'description',
        'datastore_version_id',
        'datastore_version_name',
        'datastore_name',
        'created',
        'updated',
        'allowed_updated',
        'instance_count',
        'values'
    )

    def get_parser(self, prog_name):
        parser = super(SetConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            '--parameter-group',
            metavar="<parameter_group>",
            required=True,
            help=_("Parameter group id")
        )
        parser.add_argument(
            '--name',
            metavar="<name>",
            help=_("New ParameterGroup name")
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help=_("New ParameterGroup description")
        )
        # parser.add_argument(
        #     '--datastore_type',
        #     metavar="<datastore_type>",
        #     choices=DATASTORE_TYPE_CHOICES,
        #     required=True,
        #     help=_("Datastore type")
        # )
        # parser.add_argument(
        #     '--datastore_version',
        #     metavar="<datastore_version",
        #     required=True,
        #     help=_("Datastore version")
        # )
        parser.add_argument(
            '--value',
            dest="values",
            metavar="<key=value>",
            required=True,
            action=parseractions.KeyValueAction,
            help=_("Parameter group value"
                   "(repeat option to set multiple values)")
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        config_attrs = {}

        if parsed_args.name:
            config_attrs['name'] = parsed_args.name
        if parsed_args.description:
            config_attrs['description'] = parsed_args.description

        # flatten values into the proper Configuration_attrs
        if getattr(parsed_args, 'values', None):
            config_attrs['values'] = {}
            for k, v in parsed_args.values.items():
                config_attrs['values'][k] = str(v)

        config = client.get_parameter_group(parsed_args.parameter_group)

        if not config:
            msg = (_("Failed to find Parameter Group by ID %s")
                   % parsed_args.parameter_group)
            raise exceptions.CommandError(msg)

        config = config.change_parameter_info(session=client, **config_attrs)

        data = utils.get_item_properties(
            config, self.columns,
            formatters={
                'values': format_dict,
            }
        )

        return (self.columns, data)


class DeleteConfiguration(command.Command):
    _description = _("Deletes a configuration group.")

    def get_parser(self, prog_name):
        parser = super(DeleteConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            "configuration_group",
            metavar="<configuration_group>",
            help=_("ID or name of the configuration group"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        try:
            configuration = client.find_configuration(
                parsed_args.configuration_group)
            client.delete_configuration(configuration)
        except Exception as e:
            msg = (_("Failed to delete configuration %(c_group)s: %(e)s")
                   % {'c_group': parsed_args.configuration_group, 'e': e})
            raise exceptions.CommandError(msg)


class ListDatabaseConfigurationParameters(command.Lister):

    _description = _("Lists available parameters for a configuration group.")
    columns = ['Name', 'Type', 'Min Size', 'Max Size', 'Restart Required']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseConfigurationParameters, self).\
            get_parser(prog_name)
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('Datastore version name or ID assigned'
                   'to the configuration group.')
        )
        parser.add_argument(
            '--datastore',
            metavar='<datastore>',
            default=None,
            help=_('ID or name of the datastore to list configuration'
                   'parameters for. Optional if the ID of the'
                   'datastore_version is provided.')
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError
        # db_configuration_parameters = self.app.client_manager.rds
        # if parsed_args.datastore:
        #     params = db_configuration_parameters.\
        #         parameters(parsed_args.datastore,
        #                    parsed_args.datastore_version)
        # elif utils.is_uuid_like(parsed_args.datastore_version):
        #     params = db_configuration_parameters.\
        #         parameters_by_version(parsed_args.datastore_version)
        # else:
        #     raise exceptions.NoUniqueMatch(_('The datastore name or id is'
        #                                      ' required to retrieve the'
        #                                      ' parameters for the'
        #                                      ' configuration group'
        #                                      ' by name.'))
        # for param in params:
        #     setattr(param, 'min_size', getattr(param, 'min', '-'))
        #     setattr(param, 'max_size', getattr(param, 'max', '-'))
        # params = [utils.get_item_properties(p, self.columns)
        #           for p in params]
        # return self.columns, params


class ShowDatabaseConfigurationParameter(command.ShowOne):
    _description = _("Shows details of a database configuration parameter.")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseConfigurationParameter, self).\
            get_parser(prog_name)
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('Datastore version name or ID assigned to the'
                   ' configuration group.'),
        )
        parser.add_argument(
            'parameter',
            metavar='<parameter>',
            help=_('Name of the configuration parameter.'),
        )
        parser.add_argument(
            '--datastore',
            metavar='<datastore>',
            default=None,
            help=_('ID or name of the datastore to list configuration'
                   ' parameters for. Optional if the ID of the'
                   ' datastore_version is provided.'),
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError
        # db_configuration_parameters = self.app.client_manager.database.\
        #     configuration_parameters
        # if parsed_args.datastore:
        #     param = db_configuration_parameters.get_parameter(
        #         parsed_args.datastore,
        #         parsed_args.datastore_version,
        #         parsed_args.parameter)
        # elif utils.is_uuid_like(parsed_args.datastore_version):
        #     param = db_configuration_parameters.get_parameter_by_version(
        #         parsed_args.datastore_version,
        #         parsed_args.parameter)
        # else:
        #     raise exceptions.NoUniqueMatch(_('The datastore name or id is'
        #                                      ' required to retrieve the'
        #                                      ' parameter for the'
        #                                      ' configuration group'
        #                                      ' by name.'))
        # return zip(*sorted(param._info.items()))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Config(resource.Resource):

    base_path = '/antiddos/query_config_list'

    # capabilities
    allow_list = True

    # Properties
    #: A list of traffic limited.
    #: *Type: list*
    traffic_limited_list = resource.Body('traffic_limited_list', type=list)
    #: Http limit list
    #: *Type: list*
    http_limited_list = resource.Body('http_limited_list', type=list)
    #: Connection limit list
    #: *Type: list*
    connection_limited_list = resource.Body('connection_limited_list',
                                            type=list)
    #: Extended configuration
    #: *Type: list*
    extend_ddos_config = resource.Body('extend_ddos_config',
                                       type=list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

import mock
from keystoneauth1 import adapter

from openstack.tests.unit import base
from otcextensions.sdk.css.v1 import flavor

FAKE_ID = '68d5745e-6af2-40e4-945d-fe449be00148'

EXAMPLE = {
    'versions': [
        {
            'type': 'ess',
            'version': '7.6.2',
            'flavors': [
                {
                    'cpu': 1,
                    'ram': 8,
                    'name': 'css.medium.8',
                    'region': 'eu-de',
                    'diskrange': '40,640',
                    'availableAZ': 'eu-de-01,eu-de-02,eu-de-03',
                    'flavor_id': '6b6c0bcf-750d-4f8a-b6f5-c45a143f5198',
                }
            ],
        }
    ]
}


class TestFlavor(base.TestCase):
    def setUp(self):
        super(TestFlavor, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

    def test_basic(self):
        sot = flavor.Flavor()

        self.assertEqual('/flavors', sot.base_path)
        self.assertEqual('flavors', sot.resources_key)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):
        data = EXAMPLE['versions'][0]['flavors'][0]
        sot = flavor.Flavor(**data)
        self.assertEqual(data['name'], sot.name)
        self.assertEqual(data['cpu'], sot.vcpus)
        self.assertEqual(data['ram'], sot.ram)
        self.assertEqual(data['region'], sot.region)
        self.assertEqual(data['diskrange'], sot.disk_range)
        self.assertEqual(data['flavor_id'], sot.id)

    def test_list(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = copy.deepcopy(EXAMPLE)

        self.sess.get.return_value = mock_response

        sot = flavor.Flavor()

        result = list(sot.list(self.sess))

        self.assertEqual(1, len(result))
        self.assertEqual(EXAMPLE['versions'][0]['version'], result[0].version)
        self.assertEqual(EXAMPLE['versions'][0]['type'], result[0].type)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create Auto-Scaling Group.
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

attrs = {
    "scaling_group_name": "TestGroup",
    "scaling_configuration_id": "0d138717-e79d-4fa7-b8aa-37ddec1b3ce7",
    "desire_instance_number": 0,
    "min_instance_number": 0,
    "max_instance_number": 10,
    "vpc_id": "26ca2783-dc40-4e3a-95b1-5a0756441e12",
    "available_zones": ["eu-de-01", "eu-de-03"],
    "networks": [
        {
            "id": "25d24fc8-d019-4a34-9fff-0a09fde6a9cb"
        }
    ]
}

group = conn.auto_scaling.create_group(**attrs)
print(group)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class HostType(resource.Resource):
    resource_key = ''
    resources_key = 'dedicated_host_types'
    base_path = '/availability-zone/%(availability_zone)s/dedicated-host-types'

    # capabilities
    allow_list = True

    #: Properties
    availability_zone = resource.URI('availability_zone')
    #: Specifes the DeH type
    host_type = resource.Body('host_type')
    #: Specifes the DeH name of type
    host_type_name = resource.Body('host_type_name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ResourceType(resource.Resource):
    #: Properties
    #: Maximum quota capacity
    max = resource.Body('max', type=int)
    #: Minimum quota capacity
    min = resource.Body('min', type=int)
    #: Resource quota
    quota = resource.Body('quota', type=int)
    #: Specifies the resource types
    #: Values: server_groups, replications
    type = resource.Body('type')
    #: Specifies the number of used resources
    used = resource.Body('used', type=int)


class Quota(resource.Resource):
    """SDRS Tenant Quota Resource"""
    resources_key = 'quotas'
    base_path = '/sdrs/quotas'

    # capabilities
    allow_list = True

    #: Properties
    #: Tenant resource quotas
    resources = resource.Body('resources', type=list, list_type=ResourceType)
